/****************************************************************
ConfigI.java
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Adapted by: Joel Marlow

Adapted from: ConfigClient.C, ConfigClient.h and ConfigClientMain.C
Modified: Fall 1998 (Actually it was really, really cold, and I worked long
                               hours to get this conversion done and working)

Description: This file is the new Manager Subordinate Configuration
Protocol implementation for the Client Side.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@csds.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.client;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerClientSocket;

/**
This class currently updates the configuration of any subordinates of the
localhost.  In the future it will be used for configuration of Hummers.
 */
public class ConfigI { 


  /**
  The server socket for the MessagingServer.  This may not need to be a class
  variable.
  */
  private Socket _client_socket;
  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  private String _password = new String();
  /**
  Boolean value used in order to tell if we were ever connected to the 
  database. 
  */
  private boolean _is_connected = false;
  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants = new Properties(System.getProperties());
  /**
  int representing the port number the ConfigServer should be running on.
  The value of this int is set in the Hummer.properties file and should only be
  set via the _constants Properties object.
  */
  private int _port;
  /**
  Vector used in order to store information that is to be put into the Config object
   */
  private Vector _store = new Vector();
  /**Needed for authentication*/
  HummerKeyStore _key_store;
  /**Needed for authentication*/
  PersonalConfig _personal;


  /**
  This is the main constructor for this class.  This method sets up all of
  the data that is in the Hummer.properties file that it needs.  It also 
  make sure that it will be able to use the database driver that it finds.
   */
  public ConfigI() {
    String address = new String();

  //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
      _key_store = HummerKeyStore.getInstance();
      _key_store.load(true, address);
    }
    catch (Exception e) {
      System.err.println("ConfigI: " + e);
    }
  //++Needed for authentication
    _personal = new PersonalConfig(address, null);

    try {
      _constants = StaticMethods.getHummerProperties();
      Integer value = new Integer(_constants.getProperty("port.CONFIG_PORT"));
      _port = value.intValue();

     }
    catch (FileNotFoundException fnfe) {
      System.err.println("ConfigI: Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("ConfigI: File Hummer.properties not valid.");
      System.exit(0);
    }
    catch (NumberFormatException ne) {
      System.err.println("ConfigI: Bad port.CONFIG_PORT property.");
      System.exit(0);
    }

    // initailizing values from properties file
    try {
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
        _url = _constants.getProperty("jdbc.url." + address) + nametoken.nextToken();
      }
      else {
        _url = _constants.getProperty("jdbc.url") + nametoken.nextToken();
      }

      if (_constants.containsKey("jdbc.driver." + address)) {
        _driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
        _driver = _constants.getProperty("jdbc.driver");
      }
      if (_constants.containsKey("jdbc.user." + address)) {
        _user = _constants.getProperty("jdbc.user." + address);
      }
      else {
        _user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
        _password = _constants.getProperty("jdbc.password." + address);
      }
      else {
        _password = _constants.getProperty("jdbc.password");
      }
    }
    catch (UnknownHostException uhe) {
      System.err.println("ConfigI:Constructor: " + uhe);
    }

    // Connect to DB server see if the Driver is there to use
    try {
      Class.forName (_driver);
      _is_connected = true;
    } 
    catch (ClassNotFoundException C) { 
      System.err.println ("ConfigI:Constructor: " + C);
    }
  } //ConfigI constructor

  /**
  This method opens the _client_socket to the subordinate.

  @param sub_name this is the InetAddress of the ConfigServer.
  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  
  public boolean openServer(InetAddress sub_name) {
    // Create new socket 
    try  {
      _client_socket = new Socket(sub_name, _port);
      System.out.println("Connected to chat server.");
    } 
    catch(UnknownHostException e) {
        System.err.println("ConfigI:openServer: " + e);
        System.exit(1);
    }
    catch(IOException e) {
      System.err.println("ConfigI:openServer: " + e);
      System.exit(1);
    }
    return true;
  } //openServer

  /**
  This method sends the data to the server and closes the _client_socket
  to the subordinate.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  public boolean closeServer() {
    Config config = new Config();  
  //++Needed for authentication
    Response response = new Response();
  //++Needed for authentication
    HummerClientSocket dcs = null;

    config.data = _store;
  //++This try and catch is needed for authentication
    try {
      dcs = new HummerClientSocket(_personal, _client_socket, _key_store);
      //    dcs.validation("129.101.54.53"); this should be the same as below
      dcs.validation(null);
    }
    catch (Exception e) {
      //      e.printStackTrace();
      System.err.println("ConfigI:closeServer: " + e);
      //      System.out.println("I have an error here!");
    }

  //++Needed for authentication
    response._type = Response._EOT;
  //++Needed for authentication
    response._message = config;
    // End of protocol mesage, and close out connection
  //++This try and catch is needed for authentication
    try  {
      dcs.writeResponseObject(response);
      dcs.close();
    }
    catch(UnknownHostException e) {
        System.err.println("ConfigI:closeServer:Host not found:  " + e);
        System.exit(1);
    }
    catch(IOException e) {
      System.err.println("ConfigI:closeServer: " + e);
      System.exit(1);
    }
    catch (Exception e) {
      System.err.println("ConfigI:closeServer: " + e);
    }

    return true;
  } //closeServer

  /**
  This method sends the non-inherited levels to the ConfigServer
  named in sub_name.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  
  public boolean pushLevels() {
    
    // Memory for the query we are going to build
    String query = new String ("levels delete 1");
    String start_addr = new String();
    String end_addr = new String();
    String trust = new String();
    String integrity = new String();
    String cooperation = new String();
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;

    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM levels WHERE inh=0;"); 
        _store.addElement(query);
          while (result_set.next()) {
            // Start of the SQL statement
            query = "levels insert ";         

            // SQL doesn't like ( , , ,) to fill in Null values 
            // we must explicitly say where the NULLs go like ( NULL, NULL, NULL)
            // Check to see if it is an empty value, if so put a NULL in its place
            start_addr = result_set.getString("start_addr");               
            if ( start_addr == null)
               start_addr = " NULL ";  
            else
               start_addr = "'" + start_addr + "' ";  
            end_addr = result_set.getString("end_addr");               
            if ( end_addr == null)
               end_addr = "NULL ";  
            else
               end_addr = "'" + end_addr + "' ";  
            trust = result_set.getString("trust");               
            if ( trust == null)
               trust = "NULL ";  
            else
               trust = trust + " ";  
            cooperation = result_set.getString("coop");               
            if ( cooperation == null)
               cooperation = "NULL ";  
            else
               cooperation = cooperation + " ";  
            integrity = result_set.getString("integ");               
            // end the query with 1 for the inherited value
            if ( integrity == null)
               integrity = "NULL 1";  
            else
               integrity = integrity + " 1";  

            //now put it all together into one string
            query = query + start_addr + end_addr + trust + cooperation + integrity; 

            // write the query to the server
            _store.addElement(query);
	  
          } // while 
        result_set.close();
        statement.close();
        db_connect.close();
        return true;
      } // try using the db
      catch (SQLException S) {
        System.err.println("ConfigI:pushLevels: " + S); 
        return false;
      }
    } // if (_is_connected)
    else {
      return false;
    }
  } //pushLevels


  /**
  This method sends the non-inherited filters to the ConfigServer
  named in sub_name.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */

  public boolean pushFilters() {

    // Init some memory for the SQL query
    // To make sure the tables are consistant with what is in the manager
    // Delete all inhereted values
    String query = new String("filter delete 1");
    String regular_expression = new String();
    String begin_date = new String();
    String end_date = new String();
    String begin_time = new String();
    String end_time = new String();
    String trust = new String();
    String integrity = new String();
    String cooperation = new String();
    String log = new String();
    String alert = new String();
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;

    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM filter WHERE inh=0;"); 
        _store.addElement(query);
          while (result_set.next()) {
            
            // Start of the SQL statement
            query = "filter insert ";         

            // SQL doesn't like ( , , ,) to fill in Null values 
            // we must explicitly say where the NULLs go like ( NULL, NULL, NULL)
            // Check to see if it is an empty value, if so put a NULL in its place

            // This getting one item at a time is a pain
            // there should be a better way around this problem
            // it is going to take a bit to get all the info
            // this is reason #5462 why I don't like PostgreSQL
            regular_expression = result_set.getString("expression");               
            if (regular_expression == null)
              regular_expression = "NULL ";  
            else
              regular_expression = "'" + regular_expression + "' ";  
            begin_date = result_set.getString("date_start");               
            if (begin_date == null)
              begin_date = "NULL ";  
            else
              begin_date = "'" + begin_date + "' ";   
            end_date = result_set.getString("date_end");               
            if (end_date == null)
              end_date = "NULL ";  
            else
              end_date = "'" + end_date + "' ";   
            begin_time = result_set.getString("time_start");               
            if (begin_time == null)
              begin_time = "NULL ";  
            else
              begin_time = "'" + begin_time + "' ";   
            end_time = result_set.getString("time_end");
            if (end_time == null)
              end_time = "NULL ";  
            else
              end_time = "'" + end_time + "' ";  
            trust = result_set.getString("trust");            
            if (trust == null)
              trust = "NULL ";  
            else
              trust =  trust + " ";
            cooperation = result_set.getString("coop");            
            if (cooperation == null)
              cooperation = "NULL ";               
            else
              cooperation = cooperation + " ";                
            integrity = result_set.getString("integ");            
            if (integrity == null)
              integrity = "NULL ";   
            else
              integrity = integrity + " ";    
            log =  result_set.getString("log");
            log = log + " "; 
            alert =  result_set.getString("alert");
            // end the query with 1 for the inherited value
            alert = alert + " 1";

            //now put it all together into one string
            query = query + regular_expression + begin_date + end_date + begin_time + end_time + trust + cooperation + integrity + log + alert; 

            _store.addElement(query);
          } // while
        result_set.close();
        statement.close();
        db_connect.close();
        return true;
      } // try using the db
      catch (SQLException S) {
        System.err.println("ConfigI:pushFilters: " + S); 
        return false;
      }
    } // if (_is_connected)
    else {
      return false;
    }
  } //pushFilters

  /**
  This method sends the non-inherited accept_list to the ConfigServer
  named in sub_name.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */

  public boolean pushAccept() {

    String query = new String("accept_list delete 1");    
    String filter = new String();    
    String start_addr = new String();    
    String end_addr = new String();    
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;

    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM accept_list WHERE inh=0;"); 
        _store.addElement(query);
          while (result_set.next()) {

            // Start of the SQL statement
            query = "accept_list insert ";         

            // SQL doesn't like ( , , ,) to fill in Null values 
            // we must explicitly say where the NULLs go like ( NULL, NULL, NULL)
            // Check to see if it is an empty value, if so put a NULL in its place
            filter = result_set.getString("filter");               
            if ( filter == null)
               filter = "NULL ";  
            else
               filter = "'" + filter + "' ";  
            start_addr = result_set.getString("start_addr");               
            if ( start_addr == null)
               start_addr = "NULL ";  
            else
               start_addr = "'" + start_addr + "' ";  
            end_addr = result_set.getString("end_addr");               
            // end the query with 1 for the inherited value
            if ( end_addr == null)
               end_addr = "NULL 1";  
            else
               end_addr = "'" + end_addr + "' 1";  

            //now put it all together into one string
            query = query + filter + start_addr + end_addr;
            _store.addElement(query);
          } // while
        result_set.close();
        statement.close();
        db_connect.close();
        return true;
      } // try using the db
      catch (SQLException S) {
        System.err.println("ConfigI:pushAccept: " + S); 
        return false;
      }
    } // if (_is_connected)
    else {
      return false;
    }
  } //pushAccept

  /**
  This method sends the non-inherited kill file list to the
  ConfigServer named in sub_name.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */

  public boolean pushKill() {
  
    String query = new String("kill_file delete 1");
    String start_addr = new String();
    String end_addr = new String();
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;
  
    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM kill_file WHERE inh=0;"); 
        _store.addElement(query);
          while (result_set.next()) {

            // Start of the SQL statement
            query = "kill_file insert ";         

            // SQL doesn't like ( , , ,) to fill in Null values 
            // we must explicitly say where the NULLs go like ( NULL, NULL, NULL)
            // Check to see if it is an empty value, if so put a NULL in its place
            start_addr = result_set.getString("start_addr");               
            if ( start_addr == null)
               start_addr = "NULL ";  
            else
               start_addr = "'" + start_addr + "' ";  
            end_addr = result_set.getString("end_addr");               
            // end the query with 1 for the inherited value
             if ( end_addr == null)
               end_addr = "NULL 1";  
            else
               end_addr = "'" + end_addr + "' 1";  

            //now put it all together into one string
            query = query + start_addr + end_addr;
            _store.addElement(query);
          } // while
        result_set.close();
        statement.close();
        db_connect.close();
        return true;
      } // try using the db
      catch (SQLException S) {
        System.err.println("ConfigI:pushKill: " + S); 
        return false;
      }
    } // if (_is_connected)
    else {
      return false;
    }
  } //pushKill


  /**
  This method sends the non-inherited forward_list to the
  ConfigServer named in sub_name.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */

  public boolean pushForward() {

    String query = new String("forward_list delete 1");
    String filter = new String();
    String host = new String();
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;

    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM forward_list WHERE inh=0;"); 
        _store.addElement(query);
          while (result_set.next()) {

            // Start of the SQL statement
            query = "forward_list insert ";         

            // SQL doesn't like ( , , ,) to fill in Null values 
            // we must explicitly say where the NULLs go like ( NULL, NULL, NULL)
            // Check to see if it is an empty value, if so put a NULL in its place
            filter = result_set.getString("filter");               
            if ( filter == null)
               filter = "NULL ";  
            else
               filter = "'" + filter + "' ";  
            host = result_set.getString("host");               
            if ( host == null)
               host = "NULL 1";  
            else
               host = "'" + host + "' 1";  

            //now put it all together into one string
            query = query + filter + host;
            _store.addElement(query);
          } // while
        result_set.close();
        statement.close();
        db_connect.close();
        return true;
      } // try using the db
      catch (SQLException S) {
        System.err.println("ConfigI:pushForward: " + S); 
        return false;
      }
    } // if (_is_connected)
    else {
      return false;
    }
  } //pushForward

  /**
  This mehtod sends the non-inherited Configuration to the 
  ConfigServer for each Subordinate in the DB.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */

  public boolean pushConfig() {

    InetAddress sub_addr = null;
    Connection db_connect;
    Statement statement = null;
    ResultSet result_set = null;
    Vector sub_list = new Vector();
    Enumeration enum;

    //connect to the database
    if (_is_connected) {
      try {
        db_connect = DriverManager.getConnection(_url, _user, _password);
        statement = db_connect.createStatement();
        result_set = statement.executeQuery ("select * FROM subordinates;"); 
        while (result_set.next()) {
          sub_list.addElement(result_set.getString("subord"));
        }
        result_set.close();
        statement.close();
        db_connect.close();
      } 
      catch (SQLException S) {
        System.err.println("ConfigI:pushConfig: " + S); 
        return false;
      }

      // For every subordinate send our tables
      //Need to use MetaData for retrieving max number of rows i table
      enum = sub_list.elements();
      while (enum.hasMoreElements()) {
        try {
          sub_addr = InetAddress.getByName((String)enum.nextElement());
        }
        catch(UnknownHostException e){
          System.err.println("ConfigI:pushConfig: " + e);
        }

        // Open connection to the ConfigServer running on the subord.
        if (openServer (sub_addr)) {

          // Push all our non-inherited values down 
          pushFilters();  // filters
          pushLevels();   // trust, coop, integ
          pushAccept();   // accept lists for filters
          pushForward();  // forward lists for filters
          pushKill();     // Kill file list
	  //          System.out.println("I made it before");
          closeServer();  // End connection to subord
	  //          System.out.println("I made it after");
          System.out.println("Data on " + sub_addr + " has been updated");
        }
      }
      return true;
    }
    else {
      return false;
    }
  } 

  /**
  This function creates the configuration client object and Pushes it.
  */

  public static void main(String args[]) {

    ConfigI cc = new ConfigI();
    cc.pushConfig();
  }
} // ConfigI class

