/*
Java Hummer Project
File: Message.java
Date: 11-16-98
Author: Jesse C. McConnell
Description:  The basic Message object.  Very straight forward.

Adapted from Message.h from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.dataobject;

import java.io.Serializable;
import java.util.Date;

/**
This is the basic Message Object.  When a message is set on the wire, it is
a serialization of this object.  All communication between Servers and Clients
that deal with passing Messages, are passing these.

@author Jesse C. McConnell
*/
public class Message implements Serializable{

  /* These are here for the purpose of passing a message object with
     the TIC information in it.  This information can be used in a
     variety of ways. */
  /**
  This Trust Level value is used when passing messages that around within
  the MessagingServer to other Servers (like the PeerServer).  This is a hack
  job until a better method passing this information is determined. FIX

  @serial
  */
  public String trust = new String();
  /**
  This Cooperation Level value is used when passing message around within the
  MessagingServer to other Servers (like the PeerServer).  This is a hack job
  until a better method of passing this information is determined. FIX

  @serial
  */
  public String cooperation = new String();
  /**
  This Integrity Level value is used when passing message around within the
  MessagingServer to other Servers (like the PeerServer).  This is a hack job
  until a better method of passing this information is determined. FIX

  @serial
  */
  public String integrity = new String();

/* Normal field for a message being shipped out on the wire. */

  /**
  This is the description for the Message.  It will be regexp matched to the
  filters in the MessagingServer to determine if the Message is considered
  important.

  @serial
  */
  public String description = new String();
  /**
  The creation date and time of the Message.

  @serial
  */
  public Date date_time = new Date();
  /**
  String representing the filename of the tool that generated the Message.
  This will eventually be replaced by a signature from the originating tool.

  @serial
  */
  public String filename = new String();
  /**
  String representing the originating host IP of the message.  

  @serial
  */
  public String host_IP = new String();
  /**
  String representing the tool that generated this message.  May eventually be
  the key to match the signature of the tool to.

  @serial
  */
  public String tool = new String();
  /**
  Integer representing the age of the message.  This should be set to zero when
  first sent out on the wire.  After that it should be incremented by one for
  each host it is sent to.

  @serial
  */
  public Integer message_age = new Integer("0");
};
