/*
Java Hummer Project
File: MessageFilter.java
Date: 11-18-98
Author: Jesse C. McConnell
Description: This contains several of the objects in the MessageFilters.

Adapted from MessageFilter.h from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.dataobject;

import java.lang.String;
import java.util.Vector;
import java.util.Date;

/**
The main filter object.  Contains all sorts of vectors for lists of IPs
and filters as well as the information making up a filter.
<p>
A filter is created for matching message objects to in order to determine
whether a particular Hummer instance is interested in keeping track of a
message.

@author Jesse C. McConnell
*/
public class MessageFilter extends Object {

  /**
  Regular expression which gets matched to a description in a Message Object.
  @see Message
  */
  public String regexp;
  /**
  Starting date and time of messages that we are interested in.
  */
  public Date start_datetime;
  /**
  Ending date and time of messages that we are interested in and to end filtering between.
  */
  public Date end_datetime;
  /**
  Trust Level for the filter, the messages trust level must be greater then
  or equal to this value for the message to be accepted for processing.
  */
  public String trust_level;
  /**
  Cooperation Level for the filter, the messages cooperation level must be
  greater then or equal to this value for this message to be accepted for
  processing.
  */
  public String coop_level;
  /**
  Integrity Level for the filter, the messages integrity level must be greater
  then or equal to this value for this message to be accepted for processing.
  */
  public String integ_level;
  /**
  Vector which should take HostList objects representing acceptable hosts to
  accept messages from when using this filter.
  */
  public Vector from_host_list;
  /**
  String representing a 1 or a 0 that determines whether a message matching
  this filter should be logged.
  */
  public boolean log;
  /**
  Boolean representing  true or false determining whether a message matching
  this filter should be alerted to the Interface console.
  */
  public boolean alert;
  /**
  Vector which should take HostList objects representing hosts to forward
  messages which match this filter to.
  */
  public Vector forward_host_list;
  /**
  Vector which should take PeerList objects representing peer groups to
  forward messages to.
  */
  public Vector forward_peer_list;
};

