/*
Java Hummer Project
File: Response.java
Date: 1-16-98
Author: Chad Ward
Description:  Object containing some information and a meaning behind
              the information.  Can be used for sending just
              information or sending a predefined response type
              with a message.

 Copyright (C) 1997-1998 The Hummer Project - hummer@csds.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.dataobject;

import java.io.Serializable;
import java.util.Date;
import java.util.*;

/**
This is the basic Response Object.  When a message is sent across a socket, it is
a serialization of this object with an additional type.  All communication between Servers and Clients
that deal with passing configuration information, are passing these.  Also notice that _type is a power
 of 2, so that masking can be done on the type.
@author Chad Ward
*/
public class Response implements Serializable{

	/**
	This holds the sql statements that need to be done.
	@serial
	*/
  public Vector _data = new Vector();

	/**
	Type of data that is being sent (default=_OK)
	@see Response#_command
	@see Response#_EXCEPTION
	@see Response#_OK
	@serial
	*/
  public int _type = _OK;

	/**
	Command type Response object.  The type must also be set to _OK for this to work.
	(default=0 - Invalid command) 
	@see Response#_type
	@serial
	*/
  public int _command = 0;

	/**
	Unique id given to the message being sent.  Is used to prevent replay attacks.
	In most cases will start with (new Date()).getTime(), and then incremented
	for each new message sent.
	@serial
	*/
  public long _id;


	/**
	A single piece of information that is sent should use
	this rather than the data Vector
	@serial
	*/
  public Object _message = null;


	/**
	Definitions for types of messages that contain only data being transfered.
	The types of messages are powers of two so they can masked together
	for instances that make sense (eg. message is ok, but a new signature is required)
	@see Response#_type
	*/
  public static final int _OK = 0x1;

	/**
	Type for exception message.
	@see Response#_type
	*/
  public static final int _EXCEPTION = 0x2;

	/**
	Type for a message that is the last transmission.  This value
	also has the _OK bit turned on, as well as its own unique bit.
	In other words (_EOT & _OK) == _OK.
	@see Response#_OK
	@see Response#_type
	*/
  public static final int _EOT = 0x5;

	/**
	Definitions for types of commands.
	The types of messages are powers of two so they can masked together
	for instances that make sense (eg. message is ok, but a new signature is required)
	@see Response#_command
	@see Response#_UPDATE_PUBLIC_KEY
	@see Response#_GET_HUMMER_CERTIFICATE
	@see Response#_DELETE_ALIAS
	*/
  public static final int _UPDATE_PUBLIC_KEY = 0x1,
                          _GET_HUMMER_CERTIFICATE = 0x2,
                          _SET_HUMMER_CERTIFICATE = 0x4,
                          _ADD_ALIAS = 0x8,
                          _RESET_ALIAS = 0x10,
                          _DELETE_ALIAS = 0x20,
                          _GET_ALIASES = 0x40,
                          _GET_ACCESS_VALUES = 0x80,
                          _SET_ACCESS_VALUES = 0x100,
                          _ADD_MANAGER = 0x200,
                          _ADD_PEER_MODERATOR = 0x400,
                          _RELOAD_KEY_STORE = 0x800;
};