/****************************************************************
ConfigServer.java
CS 481 - Team E
Spring 1998
Hummingbird Project - Dr. Frincke
Coded by: Jamie Marconi
Adapted by: Joel Marlow and Anu Kamath

Adapted from: ConfigServer.C and ConfigServer.h
Modified: Fall 1998 converted over to Java code
Description: This is the implementation of the configuration server.
             The server waits for clients to connect and then 
	     executes their SQL commands.
*****************************************************************/
/*
 Copyright (C) 1997-1998 The HummingBird Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.server;

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;

/**
The ConfigServer class is the main access point for ConfigI.  It will push it will change its 
configuration based on the information that it receives.  This class only listens for a 
connection.  It does not handle the information coming in.  All information handling is 
done by ClientHandler.  

@author Joel Marlow
*/

public class ConfigServer implements Runnable {


  /**
  Value for the port number that the ConfigServer will connect to.
  */
  private int _port = -1;

  /**
  Boolean value representing whether or not the MessagingServer should listen
  for additional connections.
  */
  private boolean _running = false;

  /**
  Thread used to wrap around this Runnable object.  This is done
  rather than having Messaging server extends Thread because once
  a thread completes the run() method it is considered dead...forever.
  So a new thread must be created each time that the server is started up
  again.
  */
  private Thread _wrapper_thread;

  /**
  Information about the current state of messaging server.  This will
  usually hold error information.  If MessagingServer is in a normal state,
  this will be null.
  */
  private String _status = null;

  /**
  The server socket for the ConfigServer.  This may not need to be a class
  variable.
  */
  private ServerSocket server_socket;
  /**
  System Properties object that when created contains the System.getProperties
  and gets the properties defined in Hummer.properties.
  */
  private Properties _constants;
  
  /**
  Creates the ConfigServer using the default port specified in the
  Hummer properties file.  The properties files is passed in during 
  construction of this object.

  @param constants A Hummer properties file being passed in which removes
  the extra construction time for Properties objects.
  */
  public ConfigServer(Properties constants) {
    _constants = constants;

    try{
      Integer value = new Integer(_constants.getProperty("port.CONFIG_PORT"));
      _port = value.intValue();
    }
    catch (NumberFormatException ne) {
      System.err.println("ConfigServer: Bad port.CONFIG_PORT property.");
    }
  } //ConfigServer constructor


  /**
  Returns the value of _running
  */
  public boolean isRunning() { return _running; }

  /**
  Sets the _running boolean value to false so the loop will exit.
  */
  public void setRunning(boolean running) {
    if(running) {
      if(!this._running) {
        System.out.println("Will now start config server..");
        this._running = true;
        _wrapper_thread = new Thread(this);
        _wrapper_thread.start();
      }
    }
    else {
      this._running = running;
    }
  }


	/**
	Returns the current value of _status.
	*/
  public String getStatus() { return _status; }



  /**
     The executable code.
  */
  public void run () {

    _status = null;
    try {
      server_socket = new ServerSocket(_port);
    }
    catch (IOException e) {
      _status = "ConfigServer:Could not create the server:" + e;
      System.err.println(_status);
    }
	    //ConfigServer cs = new ConfigServer();
    try { 
      while (_running) { 
		// wait to accept socket connections
        System.out.println ("\nConfigServer waiting for clients...");
        Socket client_socket = server_socket.accept(); 

		//Checks to see if we are still to be running and the state of the
		//server hasn't changed.
        if (_running) {
          ClientHandler ch = new ClientHandler (client_socket, _constants);
          ch.start();
        }
      } //while
      server_socket.close();
    }
    catch(java.net.SocketException e) {
      _status = "Socket is still pausing after closing.  Please wait.";
      System.err.println("ConfigServer:run: " + e);
    }
    catch (IOException ioe) {
      _status = "Error: " + ioe;
      System.err.println("ConfigServer:startServer: " + ioe);
    }
    catch (NullPointerException n) {
      _status = "Error: " + n;
      System.err.println ("ConfigServer:run: " + n);
    }
    _running = false;
  }

} //ConfigServer