/*
Java Hummer Project
File: InterfaceHandler.java
Date: 12-23-98
Coded by: Pete Neisen
Revised by: Jesse C. McConnell, Chad Ward
Description:  Handles requests by Interfaces.  Requests
              are first taken by InterfaceServer, and then
              passed onto this class.


 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/



package edu.uidaho.hummer.server;

import java.io.IOException;
import java.net.Socket;
import java.security.GeneralSecurityException;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;

import edu.uidaho.hummer.gui.hac.Interface;
import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.UnknownAliasException;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.socket.HummerServerSocket;
import edu.uidaho.hummer.util.socket.HummerSocketException;


/** 
Handles requests by Interfaces.  Requests
are first taken by InterfaceServer, and then
passed onto this class.
@author Pete Neisen, Jesse McConnell, Chad Ward
*/
public final class InterfaceHandler extends Thread {

  private Message _tmp_msg;
  private HummerKeyStore _key_store;  
  private HummerServerSocket _hummer_server_socket;
  private Interface _interface;

  public InterfaceHandler(Socket client_socket, HummerKeyStore key_store,
                          Interface interface_object)
  {

    this._key_store = key_store;
    this._interface = interface_object;

    try {
      _hummer_server_socket = new HummerServerSocket(null, client_socket, _key_store);
    }
    catch(IOException e) {
      System.err.println("InterfaceHandler:init: Could not create HummerServerSocket: " + e);
      return;
    }
    catch(NoSuchAlgorithmException e) {
      System.err.println("InterfaceHandler:init: Could not create HummerServerSocket: " + e);
      return;
    }
    catch(KeyStoreException e) {
      System.err.println("InterfaceHandler:init: Could not create HummerServerSocket: " + e);
      return;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("InterfaceHandler:init: Could not create HummerServerSocket: " + e);
      return;
    }
    catch(HummerSocketException e) {
      System.err.println("InterfaceHandler:init: Could not create HummerServerSocket: " + e);
      return;
    }



    try {
      _hummer_server_socket.validation(null);
    }
    catch(GeneralSecurityException e) {
      System.err.println("InterfaceHandler:init: Error during authentication with client: " + e);
      return;
    }
    catch(IOException e) {
      System.err.println("InterfaceHandler:init: Error during authentication with client: " + e);
      return;
    }
    catch(HummerSocketException e) {
      System.err.println("InterfaceHandler:init: Error during authentication with client: " + e);
      return;
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("InterfaceHandler:init: Error during authentication with client: " + e);
      return;
    }
    catch(UnknownAliasException e) {
      System.err.println("InterfaceHandler:init: Error during authentication with client: " + e);
      return;
    }

    this.start();
  }
  
  public void run() {

    String line;
    Response input;    
    try{
      input = _hummer_server_socket.readResponseObject();
      _tmp_msg = (Message)input._message;
      _interface.addToList(_tmp_msg);
    }
    catch(IOException ioe) {
      System.err.println("InterfaceHandler:run:" + ioe);
    }
    catch(GeneralSecurityException e) {
      System.err.println("InterfaceHandler:run: " + e);
    }
    catch(UnloadedKeyStoreException e) {
      System.err.println("InterfaceHandler:run: " + e);
    }
    catch(UnknownAliasException e) {
      System.err.println("InterfaceHandler:run: " + e);
    }
    catch(HummerSocketException e) {
      System.err.println("InterfaceHandler:run: " + e);
    }
    finally {
      try {
        _hummer_server_socket.close();
      }
      catch(IOException e){
        System.err.println("InterfaceHandler:run:" + e);
      }
    }
  }
}