/*
Java Hummer Project
File: ValidationTest.java
Date: 2-14-99
Author: Chad Ward
Description:  Class of public static methods.  Place to hold
		general algorithms that are not specific to any
		single class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test;

import java.util.*;
import java.io.*;
import java.net.*;
import java.security.*;
import java.security.cert.*;

import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.util.socket.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.test.client.*;
import edu.uidaho.hummer.test.dataobject.*;
import edu.uidaho.hummer.test.util.*;
import edu.uidaho.hummer.test.util.socket.*;
import edu.uidaho.hummer.test.server.ConfigServerTest;

/**
A specialized class for running automated tests validation objects.
@author Chad Ward
*/

public final class ValidationTest {

  public static void main(String[] argv) throws Exception {
    ValidationTest test = new ValidationTest();
    String usage = "======================================================================\n" +
                   "Unknown command.  Usage: \n" +
                   "  java edu.uidaho.hummer.test.ValidationTest <command|name class_name>\n" +
                   "======================================================================\n\n" +
                   "Commands: \n" +
                   "   'joel'\t- ConfigServer\n" +
                   "   'all'\t- All of current test classes\n" +
                   "   'sock'\t- HummerServerSocket & HummerClientSocket\n" +
                   "   'ssock'\t- HummerServerSocket\n" +
                   "   'csock'\t- HummerClientSocket\n" +
                   "   'ratings'\t- Tests sizes of objects created and times for creation\n" +
                   "----------------------------------------------------------------------\n\n" +
                   "Example of using \"name\" tag:\n" +
                   "  java edu.uidaho.hummer.test.ValidationTest name util.HummerKeyStore\n\n" +
                   "\tNote: This works only when there already exists a Test class called\n\t" +
                   "edu.uidaho.hummer.test.util.HummerKeyStoreTest\n";


    if(argv.length < 1) {
      System.out.println("\n" + usage + "\n");
      System.exit(-1);
    }
    else {
      if(argv[0].equals("test")) {
        test.testAll();
      }
      else if(argv[0].equals("joel")) {
        test.testConfigServer();
      }
      else if(argv[0].equals("all")) {
        test.testAll();
      }
      else if(argv[0].equals("hum")) {
        test.testHummerClasses();
      }
      else if(argv[0].equals("sock")) {
        test.testSocket();
      }
      else if(argv[0].equals("csock")) {
        test.testClientSocket();
      }
      else if(argv[0].equals("ssock")) {
        test.testServerSocket();
      }
      else if(argv[0].equals("name") && argv.length > 1) {
        Class test_object_class = Class.forName("edu.uidaho.hummer.test." + argv[1] + "Test");
        java.lang.reflect.Constructor test_object_constructor = test_object_class.getConstructor(new Class[0]);
        Test test_object = (Test)(test_object_constructor.newInstance(new Object[0]));
        Vector test_suite = new Vector();
        test_suite.add(test_object);
        Test.runTestSuite(test_suite);
      }
      else if(argv[0].equals("ratings")) {
        test.performance();
      }
      else {
        System.out.println("\n" + usage + "\n");
      }
    }
  }

  public void testAll() throws Exception {
    Vector test_suite = new Vector();

    test_suite.add(new StaticMethodsTest());
    test_suite.add(new HummerKeyStoreTest());
    test_suite.add(new HummerServerSocketTest());
    test_suite.add(new HummerClientSocketTest());
    test_suite.add(new PersonalConfigTest());
    test_suite.add(new NewUserValidatorTest());
    test_suite.add(new HummerCertificateTest());
    test_suite.add(new HummerSignedObjectTest());
    //phobos = 129.101.54.53 this configuration sever needs to be up
    test_suite.add(new ConfigServerTest("129.101.54.53", false));

    Test.runTestSuite(test_suite, System.out, System.err);
  }

  public void testHummerClasses() throws Exception {
    Vector test_suite = new Vector();
    test_suite.add(new ConfigITest());
    Test.runTestSuite(test_suite);
  }

  public void testConfigServer() throws Exception {
    Vector test_suite = new Vector();
    test_suite.add(new ConfigServerTest("129.101.54.54", true));
    Test.runTestSuite(test_suite, System.out, System.err);
  }
	/**

	*/
  public void performance() throws Exception {
    this.signatureTimes();
    this.sealedObjectTimes();
//    this.accessTableSize();
  }

  public void testValidationClasses() throws Exception {
    Vector test_suite = new Vector();
    PrintStream file_stream = new PrintStream(new FileOutputStream("validation_test.dat"));
    test_suite.add(new HummerKeyStoreTest(file_stream, System.err));
    Test.runTestSuite(test_suite);
  }

  public void testSocket() throws Exception {
    Vector test_suite = new Vector();
    PrintStream file_stream = new PrintStream(new FileOutputStream("socket_test.dat"));
    test_suite.add(new HummerServerSocketTest());
    test_suite.add(new HummerClientSocketTest());
    Test.runTestSuite(test_suite);
  }

  public void testServerSocket() throws Exception {
    Vector test_suite = new Vector();
    PrintStream file_stream = new PrintStream(new FileOutputStream("server_socket_test.dat"));
    test_suite.add(new HummerServerSocketTest());
    Test.runTestSuite(test_suite);
  }

  public void testClientSocket() throws Exception {
    Vector test_suite = new Vector();
    PrintStream file_stream = new PrintStream(new FileOutputStream("client_socket_test.dat"));
    test_suite.add(new HummerClientSocketTest());
    Test.runTestSuite(test_suite);
  }
	/**
	
	*/

  public void sealedObjectTimes() throws Exception {
		/*
		To get an accurate measurement of the times, the internal signing engine must be initialized
		If this is not done, then it will take about 2000 ms to sign a single byte.
		*/
    int number_bytes = 0, delay = 0;
    long begin_time = 0, end_time = 0;
    byte[] data;
    KeyPair key_pair1 = StaticMethods.generateKeyPair(), key_pair2 = StaticMethods.generateKeyPair();
    Signature signing_signature_object = StaticMethods.getSigningSignatureObject(key_pair1.getPrivate());
    HummerSignedObject current_signed_object = new HummerSignedObject(new byte[2], signing_signature_object);
    System.out.println("HummerSignedObject creation times");
    System.out.flush();
    for(int i = 0; i < 7; i++) {
      number_bytes = (int)(Math.pow((double)10, (double)i));
      data = new byte[number_bytes];
      begin_time = (new Date()).getTime();
      current_signed_object = new HummerSignedObject(data, signing_signature_object);
      end_time = (new Date()).getTime();
      System.out.println("\t" + number_bytes + " bytes - " + ((end_time - begin_time) - delay) + " ms.");
      System.out.flush();
    }
    System.out.println("Times for signing of PublicKeys (" +
                       "size of PublicKey: " + StaticMethods.getBytes(key_pair2.getPublic()).length +
                       " bytes)in a Vector:\nNumber of keys\nTimes" );
    Vector key_vector = new Vector();
    begin_time = (new Date()).getTime();
    current_signed_object = new HummerSignedObject(key_vector, signing_signature_object);
    end_time = (new Date()).getTime();
    System.out.println(key_vector.size() + "\t" + ((end_time - begin_time) - delay) + " ms.");
    for(int i = 0; i < 10; i++) {
      key_vector.add(key_pair2.getPublic());
      begin_time = (new Date()).getTime();
      current_signed_object = new HummerSignedObject(key_vector, signing_signature_object);
      end_time = (new Date()).getTime();
      System.out.println(key_vector.size() + "\t" + ((end_time - begin_time) - delay) + " ms.");
   }
    
  }

  public void signatureTimes() throws Exception {
    long time = 0;
    KeyPair key_pair = StaticMethods.generateKeyPair();
    System.out.println("Verifying Signature object creation times");
    for(int i = 1; i <= 20; i++) {
      time = System.currentTimeMillis();
      for(int j = 0; j < i; j++) {
        StaticMethods.getVerifyingSignatureObject(key_pair.getPublic());
      }
      time = System.currentTimeMillis() - time;
      System.out.println(i + "\t objects:\t:" + time);
    }
    System.out.println("\nSigning Signature object creation times");
    for(int i = 1; i <= 20; i++) {
      time = System.currentTimeMillis();
      for(int j = 0; j < i; j++) {
        StaticMethods.getSigningSignatureObject(key_pair.getPrivate());
      }
      time = System.currentTimeMillis() - time;
      System.out.println(i + "\t objects:\t:" + time);
    }
  }

  public void accessTableSize() throws Exception {
    String root_name = "name_";
    Hashtable access_table = new Hashtable();
    System.out.println("Access table sizes:\nString Key Length\tNumber of Values\tSize (bytes)");
    System.out.println(root_name.length() + "\t\t\t" + access_table.size() + "\t\t\t" + StaticMethods.getBytes(access_table).length);
    for(int i = 0; i < 10; i++) {
      for(int j = 0; j < 5; j++) {
        access_table.put(root_name + j, Boolean.TRUE);
        System.out.println(root_name.length() + "\t\t\t" + access_table.size() + "\t\t\t" +
                           StaticMethods.getBytes(access_table).length);
      }
      root_name += "" + i;
      System.out.println(root_name.length() + "\t\t\t" + access_table.size() + "\t\t\t" +
                         StaticMethods.getBytes(access_table).length);
    }
  }

}