/*
Java Hummer Project
File: HummerSignedObjectTest.java
Date: 3-31-99
Author: Chad Ward
Description:  Tests HummerSignedObject class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.dataobject;

import java.io.*;
import java.security.*;
import java.util.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.test.Test;

/**
Tests HummerSignedObject class.
@author Chad Ward
*/
public class HummerSignedObjectTest extends Test {

  public HummerSignedObjectTest() { super(); }
  public HummerSignedObjectTest(PrintStream out, PrintStream err) { super(out, err); }

        /**
        Main method so that this test can be run from the command line
        */
  public static void main(String[] argv) throws Exception {
    java.io.PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new java.io.PrintStream(new java.io.FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new java.io.PrintStream(new java.io.FileOutputStream(argv[1]));
    }
    Test test_object = new HummerSignedObjectTest(std_out, err_out);
    java.util.Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }

	/**
	Initializes the _test_class_name variable.
	Grabs the name of the test class using the Class.getName() method and then removing the last 4
	characters in the name ("Test")
	*/
  protected void setTestClassName() {
    _test_class_name = "HummerSignedObject";
  }

  protected void testDefinition() throws Exception {
    HummerSignedObject signed_object;
    String string_object = "The string object";
    KeyPair signature_key_pair = StaticMethods.generateKeyPair(), test_key_pair = StaticMethods.generateKeyPair();
    Signature sign = Signature.getInstance("SHA1withDSA"),
              correct_verify = Signature.getInstance("SHA1withDSA"),
              incorrect_verify = Signature.getInstance("SHA1withDSA");
    sign.initSign(signature_key_pair.getPrivate());
    byte[] bytes = {(byte)0, (byte)1, (byte)2, (byte)3};
    sign.update(bytes);
    byte[] signature_bytes = sign.sign();
    correct_verify.initVerify(signature_key_pair.getPublic());
    correct_verify.update(bytes);
    incorrect_verify.initVerify(test_key_pair.getPublic());
    signed_object = (HummerSignedObject)(runConstructorTest("Null object", null, false, null, sign)[2]);
    runSimpleTest("SignedObject with null object", "Is invalid with correct signature key", !signed_object.verify(correct_verify));
    runSimpleTest("SignedObject with null object", "Is invalid with incorrect signature key", !signed_object.verify(incorrect_verify));
    runSimpleTest("SignedObject with null object", "Object in signed object is null", signed_object.getObject() == null);

    signed_object = (HummerSignedObject)(runConstructorTest("String object", null, false, string_object, sign)[2]);
    runSimpleTest("SignedObject with String object", "Is valid with correct signature key", signed_object.verify(correct_verify));
    runSimpleTest("SignedObject with String object", "Is invalid with incorrect signature key",
                  !signed_object.verify(incorrect_verify));
    runSimpleTest("SignedObject with String object", "Object in signed object is null",
                  string_object.equals(signed_object.getObject()));

    signed_object = (HummerSignedObject)(runConstructorTest("KeyPair object", null,
                                                            false, test_key_pair, sign)[2]);
    runSimpleTest("SignedObject with KeyPair object", "Is valid with correct signature key", signed_object.verify(correct_verify));
    runSimpleTest("SignedObject with KeyPair object", "Is invalid with incorrect signature key",
                  !signed_object.verify(incorrect_verify));
    KeyPair held_key_pair = (KeyPair)(signed_object.getObject());
    runSimpleTest("SignedObject with KeyPair object", "Object in signed object is null",
                  held_key_pair.getPublic().equals(test_key_pair.getPublic()) &&
                  held_key_pair.getPrivate().equals(test_key_pair.getPrivate()));
    ByteArrayOutputStream byte_out_stream = new ByteArrayOutputStream();
    ObjectOutputStream out_stream = new ObjectOutputStream(byte_out_stream);
    out_stream.writeObject(signed_object);
    bytes = byte_out_stream.toByteArray();
    ByteArrayInputStream byte_in_stream = new ByteArrayInputStream(bytes);
    ObjectInputStream in_stream = new ObjectInputStream(byte_in_stream);
    HummerSignedObject new_signed_object = (HummerSignedObject)(in_stream.readObject());
    runSimpleTest("After serialization and deserialization",
                  "Both contained objects are equal",
                  Arrays.equals(((KeyPair)new_signed_object.getObject()).getPublic().getEncoded(),
                                ((KeyPair)signed_object.getObject()).getPublic().getEncoded()));
    System.out.println("new object: " + new_signed_object.getObject() + "\nold object: " + signed_object.getObject());
    runSimpleTest("After serialization and deserialization",
                  "Signature bytes are equal",
                  Arrays.equals(new_signed_object.getSignature(), signed_object.getSignature()));
    runSimpleTest("After serialization and deserialization",
                  "Is still valid with correct Signature", new_signed_object.verify(correct_verify));
  }
}