/*
Java Hummer Project
File: HummerKeyStoreTest.java
Date: 2-17-99
Author: Chad Ward
Description:  Tests StaticMethods class.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.test.util;

import java.security.*;
import java.util.*;
import java.io.*;
import edu.uidaho.hummer.dataobject.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.test.Test;

public class HummerKeyStoreTest extends Test {

  public HummerKeyStoreTest() { super(); }
  public HummerKeyStoreTest(PrintStream out, PrintStream err) { super(out, err); }

        /**
        Main method so that this test can be run from the command line
        */
  public static void main(String[] argv) throws Exception {
    PrintStream std_out = System.out, err_out = System.err;
    if(argv.length > 0) {
      std_out = new PrintStream(new FileOutputStream(argv[0]));
    }
    if(argv.length > 1) {
      err_out = new PrintStream(new FileOutputStream(argv[1]));
    }
    Test test_object = new HummerKeyStoreTest(std_out, err_out);
    Vector test_suite = new Vector();
    test_suite.add(test_object);
    Test.runTestSuite(test_suite);
  }

	/**
	Initializes the _test_class_name variable.
	Grabs the name of the test class using the Class.getName() method and then removing the last 4
	characters in the name ("Test")
	*/
  protected void setTestClassName() {
    _test_class_name = "HummerKeyStore";
  }

  protected void testDefinition() throws Exception {

    String user1_pass = "user1_pass", user2_pass = "user2_pass",
           peer1_pass = "peer1_pass", peer2_pass = "peer2_pass",
           server1_pass = "server1_pass", server2_pass = "server2_pass", admin_pass = "admin_pass";
    KeyPair user1_key_pair = StaticMethods.generateKeyPair(),
            user2_key_pair = StaticMethods.generateKeyPair(),
            peer1_key_pair = StaticMethods.generateKeyPair(),
            server1_key_pair = StaticMethods.generateKeyPair(), server2_key_pair = StaticMethods.generateKeyPair();
    DefaultKeyStore key_store1 = null, key_store2 = null;
    Signature verifying_signature_object, signing_signature_object;
    Hashtable access_table1 = new Hashtable(), access_table2 = new Hashtable(),
              access_table3 = new Hashtable(), access_table4 = new Hashtable();
    access_table1.put("table_one_value", new Boolean(true));
    access_table1.put("table_three_value", new Boolean(false));
    access_table2.put("table_two_value", new Boolean(true));
    access_table2.put("table_four_value", new Boolean(false));
    access_table3.put("table_three_value", new Boolean(true));
    access_table3.put("table_one_value", new Boolean(false));
    access_table4.put("table_four_value", new Boolean(true));
    access_table2.put("table_two_value", new Boolean(false));
		/* ***********************
		Constructor testing
		*************************/
    newMethod("getInstance");
    key_store1 = (DefaultKeyStore)HummerKeyStore.getInstance();

		/* ***********************
		load testing (admin account)
    		*************************/
    newMethod("load (null InputStream)");
    runExceptionTest("Called addServerAlias method with un-loaded HummerKeyStore",
                     key_store1, "addServerAlias", "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     "something", new byte[2], Boolean.TRUE, access_table1);

    runExceptionTest("Called addUserAlias method with un-loaded HummerKeyStore", key_store1, "addUserAlias", 
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     "something", new byte[2], access_table1);

    runExceptionTest("Called addPeerAlias method with un-loaded HummerKeyStore", key_store1, "addPeerAlias", 
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     "something", new byte[2]);

    runExceptionTest("Called contains method with un-loaded HummerKeyStore", key_store1, "contains",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called removeAlias method with un-loaded HummerKeyStore", key_store1, "removeAlias",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called equalAccessTable method with un-loaded HummerKeyStore", key_store1, "equalAccessTable",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", access_table1);

    runExceptionTest("Called generateHummerCertificate method with un-loaded HummerKeyStore",
                     key_store1, "generateHummerCertificate",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called getAccessTable method with un-loaded HummerKeyStore", key_store1, "getAccessTable",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called getAliases method with un-loaded HummerKeyStore", key_store1, "getAliases",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true);

    runExceptionTest("Called getDigest method with un-loaded HummerKeyStore", key_store1,
                     "getDigest",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", null, null);

    runExceptionTest("Called getHummerCertificate method with un-loaded HummerKeyStore", key_store1,
                     "getHummerCertificate",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called getPersonalHummerCertificate method with un-loaded HummerKeyStore", key_store1,
                     "getPersonalHummerCertificate",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called getPersonalKeyPair method with un-loaded HummerKeyStore", key_store1,
                     "getPersonalKeyPair",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true);

    runExceptionTest("Called getPersonalKeyPair(int,Object) method with un-loaded HummerKeyStore",
                     key_store1, "getPersonalKeyPair",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     new Integer(HummerKeyStore._BACKUP_TIMESTAMP_INDEX), new Long(34L));

    runExceptionTest("Called getPersonalKeyPair(String) method with un-loaded HummerKeyStore", key_store1,
                     "getPersonalKeyPair",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     "something");

    runExceptionTest("Called getKey method with un-loaded HummerKeyStore", key_store1,
                     "getKey",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     new Integer(0), "something");

    runExceptionTest("Called hasUpdatedHummerCertificate method with un-loaded HummerKeyStore", key_store1,
                     "hasUpdatedHummerCertificate",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called initializeAccount method with un-loaded HummerKeyStore",
                     key_store1, "initializeAccount",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true,
                     "something", new byte[2], new byte[2], user1_key_pair.getPublic());

    runExceptionTest("Called isCertificateHonored method with un-loaded HummerKeyStore",
                     key_store1, "isCertificateHonored",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called isNewAccount method with un-loaded HummerKeyStore",
                     key_store1, "isNewAccount",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something");

    runExceptionTest("Called resetAlias method with un-loaded HummerKeyStore",
                     key_store1, "resetAlias",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", null);

    runExceptionTest("Called setAccessValue method with un-loaded HummerKeyStore",
                     key_store1, "setAccessValue",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", "unknown", Boolean.FALSE);

    runExceptionTest("Called setAccessTable method with un-loaded HummerKeyStore",
                     key_store1, "setAccessTable",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", new Hashtable());


    runExceptionTest("Called setHummerCertificate method with un-loaded HummerKeyStore",
                     key_store1, "setHummerCertificate",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, "something", null);

    runExceptionTest("Called setKey method with un-loaded HummerKeyStore",
                     key_store1, "setKey",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException",
                     true, new Integer(0), "something", user1_key_pair.getPublic());

    runExceptionTest("Called size method with un-loaded HummerKeyStore", key_store1, "size",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true);

    runExceptionTest("Called updateKeyPair method with un-loaded HummerKeyStore",
                     key_store1, "updateKeyPair",
                     "edu.uidaho.hummer.util.UnloadedKeyStoreException", true, Boolean.FALSE);

    runExceptionTest("Called load with non-null admin password (admin_pass)", key_store1, "load",
                     null, false, admin_pass.getBytes());

		/* ***********************
		size testing
		*************************/
    newMethod("size(only admin account present)");
    runSimpleTest("Only admin alias in HummerKeyStore (none added explicitly)", "size()=1", key_store1.size() == 1);

		/* ***********************
		addServerAlias testing
		*************************/
    newMethod("addServerAlias");
    runExceptionTest("Trusted server equals true and null password", key_store1,
                     "addServerAlias", null, false, "server2", null, Boolean.TRUE, access_table2);
    try {
      runSimpleTest("getAliasType method called" , "equals AliasInfo._SERVER_ALIAS_TYPE",
                    key_store1.getAliasType("server2") == AliasInfo._SERVER_ALIAS_TYPE);
    }
    catch(Exception e) {}
    runExceptionTest("Trusted server equals true and null password and null access_table",
                     key_store1, "addServerAlias", null, false, "server_empty1", null, Boolean.TRUE, null);
    runExceptionTest("Trusted server equals true and null password and null access_table",
                     key_store1, "addServerAlias", null, false, "server_empty2", null, Boolean.TRUE, null);
    runExceptionTest("Did not throw exception for non-trusted server type and non-null password",
                     key_store1, "addServerAlias", null, false,"server1", server1_pass.getBytes(),
                     Boolean.FALSE, access_table1);
    try {
      runSimpleTest("getAliasType method called", "AliasInfo._SERVER_ALIAS_TYPE",
                    key_store1.getAliasType("server1") == AliasInfo._SERVER_ALIAS_TYPE);
    }
    catch(Exception e) {}
    runExceptionTest("Duplicate alias and null password (server1)", key_store1, "addServerAlias",
                     null, true,"server1", null, Boolean.TRUE, access_table1);
    runExceptionTest("Empty alias (null password)", key_store1, "addServerAlias",
                     null, true, "", null, Boolean.TRUE, access_table1);
    runExceptionTest("Null alias (null password)", key_store1, "addServerAlias",
                     null, true, null, null, Boolean.TRUE, access_table1);
    runSimpleTest("size method called", "5", key_store1.size() == 5);

		/* ***********************
		addUserAlias testing
		*************************/
    newMethod("addUserAlias");
    runExceptionTest("Non-empty alias and non-null password", key_store1, "addUserAlias",
                     null, false, "user1", user1_pass.getBytes(), access_table3);
    try {
      runSimpleTest("getAliasType method called", "AliasInfo._USER_ALIAS_TYPE",
                    key_store1.getAliasType("user1") == AliasInfo._USER_ALIAS_TYPE);
    }
    catch(Exception e) {}
    runExceptionTest("Null password",
                     key_store1, "addUserAlias", null, false, "Empty User1", null, new Hashtable());
    runExceptionTest("Null access table",
                     key_store1, "addUserAlias", "edu.uidaho.hummer.util.UnknownAliasException",
                     false, "Empty User2", "empty_pass".getBytes(), null);
    runExceptionTest("Non-empty alias and non-null password", key_store1, "addUserAlias",
                     null, false, "user2",
                     user2_pass.getBytes(), access_table4);
    try {
      runSimpleTest("getAliasType method called", "AliasInfo._USER_ALIAS_TYPE",
                    key_store1.getAliasType("user2") == AliasInfo._USER_ALIAS_TYPE);
    }
    catch(Exception e) {}

    runExceptionTest("Duplicate alias (null password)", key_store1, "addUserAlias",
                     "java.security.KeyStoreException", true, "user1", null, access_table3);
    runExceptionTest("Empty alias (null password)", key_store1, "addUserAlias",
                     "java.security.KeyStoreException", true, "", null, access_table3);
    runExceptionTest("Null alias (null password)", key_store1, "addUserAlias",
                     "java.security.KeyStoreException", true, null, null, access_table3);
    runSimpleTest("size method called", "9", key_store1.size() == 9);


		/* ***********************
		addPeerAlias testing
		*************************/
    newMethod("addPeerAlias");
    runExceptionTest("Non-empty alias and non-null password(peer1)", key_store1, "addPeerAlias",
                     null, false, "peer1", peer1_pass.getBytes());
    runSimpleTest("Non-empty alias and non-null password(peer1)",
                  "newAccount(peer1) returns true", key_store1.isNewAccount("peer1"));
    try {
      runSimpleTest("getAliasType method called", "AliasInfo._PEER_ALIAS_TYPE",
                    key_store1.getAliasType("peer1") == AliasInfo._PEER_ALIAS_TYPE);
    }
    catch(Exception e) {}
    runExceptionTest("Null password (Empty Peer1)",
                     key_store1, "addPeerAlias", null, false, "Empty Peer1", null);
    runExceptionTest("Non-empty alias and non-null password (peer2)", key_store1, "addPeerAlias",
                     null, false, "peer2", peer2_pass.getBytes());
    runSimpleTest("Non-empty alias and non-null password(peer2)",
                  "newAccount(peer2) returns true", key_store1.isNewAccount("peer2"));
    try {
      runSimpleTest("getAliasType method called", "AliasInfo._PEER_ALIAS_TYPE",
                    key_store1.getAliasType("peer2") == AliasInfo._PEER_ALIAS_TYPE);
    }
    catch(Exception e) {}

    runExceptionTest("Duplicate alias (peer1, null password)", key_store1, "addPeerAlias",
                     null, true, "peer1", null);
    runExceptionTest("Empty alias (\"\", null password)", key_store1, "addPeerAlias",
                     null, true, "", null);
    runExceptionTest("Null alias (null password)", key_store1, "addPeerAlias",
                     null, true, null, null);
    runSimpleTest("size method called(peer1, peer2, Empty Peer1)", "12", key_store1.size() == 12);

		/* ***********************
		contains testing
		*************************/
    newMethod("contains");

    runExceptionTest("Unknown user when calling removeAlias", key_store1, "removeAlias",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling equalAccessTable", key_store1, "equalAccessTable",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", null);

    runExceptionTest("Unknown user when calling generateHummerCertificate", 
                     key_store1, "generateHummerCertificate",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling getAccessTable", key_store1, "getAccessTable",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling getDigest", key_store1, "getDigest",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", null, null);

    runExceptionTest("Unknown user when calling getHummerCertificate", 
                     key_store1, "getHummerCertificate",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling getPersonalHummerCertificate",
                     key_store1, "getPersonalHummerCertificate",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling getKey",
                     key_store1, "getKey",
                     "edu.uidaho.hummer.util.UnknownAliasException", true,
                     new Integer(0), "Unknown");

    runExceptionTest("Unknown user when calling hasUpdatedHummerCertificate",
                     key_store1, "hasUpdatedHummerCertificate",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling initializeAccount",
                     key_store1, "initializeAccount",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", null, null, null);

    runExceptionTest("Unknown user when calling isCertificateHonored",
                     key_store1, "isCertificateHonored",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling isNewAccount",
                     key_store1, "isNewAccount",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown");

    runExceptionTest("Unknown user when calling resetAlias",
                     key_store1, "resetAlias",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", null);

    runExceptionTest("Unknown user when calling setAccessTable",
                     key_store1, "setAccessTable",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", new Hashtable());

    runExceptionTest("Unknown user when calling setAccessValue",
                     key_store1, "setAccessValue",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", "Something", Boolean.FALSE);


    runExceptionTest("Unknown user when calling setHummerCertificate",
                     key_store1, "setHummerCertificate",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "Unknown", null);

    runExceptionTest("Unknown user when calling setKey",
                     key_store1, "setKey",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, new Integer(0),
                     "Unknown", null);

    boolean has_caught_exception = ((Boolean)(runExceptionTest("Did not throw exception for known alias (user1)",
                                            key_store1, "contains", null, false, "user1")[0])).booleanValue();
    has_caught_exception = has_caught_exception ||
                           ((Boolean)(runExceptionTest("Did not throw exception for unknown alias", key_store1,
                                            "contains", null, false, "unknown-user")[0])).booleanValue();
    has_caught_exception = has_caught_exception ||
                           ((Boolean)(runExceptionTest("Did not throw exception for empty alias", key_store1,
                                            "contains", null, false, "")[0])).booleanValue();
    has_caught_exception = has_caught_exception ||
                           ((Boolean)(runExceptionTest("Did not throw exception for null alias", key_store1,
                                            "contains", null, false, null)[0])).booleanValue();
		/*
		If there has been no exceptions thrown from contains, then more meaningful
		tests can be done on it...like whether it's actually returning the correct answer
		*/
    if(!has_caught_exception) {
      runSimpleTest("admin", "boolean value true", key_store1.contains("admin"));
      runSimpleTest("Known alias (user1)", "boolean value true", key_store1.contains("user1"));
      runSimpleTest("Unknown alias (unknown-alias)", "boolean value false", !key_store1.contains("unknown-user"));
      runSimpleTest("Empty string", "false", !key_store1.contains(""));
      runSimpleTest("null", "false", !key_store1.contains(null));
    }

		/* ***********************
		removeAlias testing
		*************************/
    newMethod("removeAlias");

    runExceptionTest("admin", key_store1, "removeAlias",
                     null, true, "admin");
    runExceptionTest("Known alias", key_store1, "removeAlias",
                     null, false, "server2");
    runSimpleTest("contains called with alias that was deleted (server2)",
                  "false", !key_store1.contains("server2"));
    runSimpleTest("size method called", "11", key_store1.size() == 11);
    try {
      key_store1.addServerAlias("server2", server2_pass.getBytes(), true, access_table2);
    }
    catch(Exception e) {}
    runSimpleTest("After adding alias back (server2), call contains method", "true", key_store1.contains("server2"));

    
		/* ***********************
		equalAccessTable testing
		*************************/
    newMethod("equalAccessTable");
    Boolean equal = (Boolean)(runExceptionTest("admin account with empty access table", key_store1,
                                               "equalAccessTable", null, false, "admin", new Hashtable())[2]);
    runSimpleTest("admin account with null access table", "boolean true", equal.booleanValue());
    runSimpleTest("Access table equal to access table for alias (user1)", "true",
                  key_store1.equalAccessTable("user1", access_table3));
    runSimpleTest("Access table differnent from access table for alias(server1)", "false",
                  !key_store1.equalAccessTable("server1", access_table4));


		/* ***********************
		initializeAccount testing
		*************************/
    newMethod("initializeAccount");
    note("User Account (user1)");
    boolean caught_exception = false;
    byte[] random_timestamp = StaticMethods.getRandomTimestampBytes();
    byte[] password_digest1 = StaticMethods.getDigest(user1_pass.getBytes(),
                                                      random_timestamp, user1_key_pair.getPublic()),
           password_digest1_w = StaticMethods.getDigest("wrong".getBytes(),
                                                        random_timestamp, user1_key_pair.getPublic());
    runExceptionTest("Known user alias with wrong password", key_store1,
                     "initializeAccount", "java.security.KeyStoreException", true,
                     "user1", password_digest1_w, random_timestamp, user1_key_pair.getPublic());
    runExceptionTest("Known user alias with null public_key", key_store1,
                     "initializeAccount", "java.security.KeyStoreException", true,
                     "user1", password_digest1, random_timestamp, null);
    runExceptionTest("Null alias (initialize account for user)", key_store1,
                     "initializeAccount", "edu.uidaho.hummer.util.UnknownAliasException", true,
                     null, password_digest1, random_timestamp, user1_key_pair.getPublic());
    runExceptionTest("Null password", key_store1,
                     "initializeAccount", "java.lang.NullPointerException", true,
                     "user1", null, random_timestamp, user1_key_pair.getPublic());

    HummerCertificate user1_recieved_certificate = 
             (HummerCertificate)(runExceptionTest("Valid parameters (alias-\"user1\", user1_key_pair.PublicKey)",
                                                   key_store1, "initializeAccount", null, false,
                                                   "user1", password_digest1, random_timestamp,
                                                   user1_key_pair.getPublic())[2]);
    runSimpleTest("user1 (above initialized account alias)",  "No longer considered a \"new\" account",
                  !key_store1.isNewAccount("user1"));
    runSimpleTest("Above returned HummerCertificate", "HummerCertificate is not null",
                  user1_recieved_certificate != null);
    runExceptionTest("Alias of account already initialized",
                     key_store1, "initializeAccount", null, true,
                     "user1", password_digest1,
                     random_timestamp, user1_key_pair.getPublic());

    if(user1_recieved_certificate != null) {
      runSimpleTest("PublicKey in certificate equals PublicKey of \"user1\"", "true", 
                   user1_recieved_certificate.getPublicKey().equals(user1_key_pair.getPublic()));
      runSimpleTest("HummerCertificate.isExpired for newly created HummerCertificate",
                    "false",!user1_recieved_certificate.isExpired());
      runSimpleTest("HummerCertificate.isValid for newly created HummerCertificate", "true",
                    user1_recieved_certificate.isValid(key_store1.getPersonalVerifySignature()));
    }
    caught_exception = !((Boolean)(runExceptionTest("Did not throw exception when " +
                                         "retrieving key (user)",
                                         key_store1, "getKey",
                                         null, false, new Integer(AliasInfo._PUBLIC_KEY_TYPE),
                                         "user1")[0])).booleanValue();
    if(!caught_exception) {
      runSimpleTest("key_store1.getPublicKey(\"user1\")", "PublicKey equal to user1 key pair",
                    key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "user1").equals(user1_key_pair.getPublic()));
    }
    runSimpleTest("Run equalAccessTable method with \"user1\"", "true",
                 key_store1.equalAccessTable("user1", access_table3));
    boolean passed = false;
		/*
		The following has not been tested yet, so exceptions should be trapped just in case
		*/
    try {
      passed = key_store1.getPersonalKeyPair("user1").equals(key_store1.getPersonalKeyPair());
    }
    catch(Exception e) {}
    runSimpleTest("Compare getPersonalKeyPair with \"user1\" " +
                  "with current key_store KeyPair", "true", passed);


		/* ****************************
		Test initializeAccount for peer aliases
		*********************************/
    note("Peer Account (peer1)");
    caught_exception = false;
    password_digest1 = StaticMethods.getDigest(peer1_pass.getBytes(),
                                               random_timestamp, peer1_key_pair.getPublic());
    password_digest1_w = StaticMethods.getDigest("wrong".getBytes(),
                                                 random_timestamp, peer1_key_pair.getPublic());
    runExceptionTest("Known user alias with wrong password", key_store1,
                     "initializeAccount", "java.security.KeyStoreException", true,
                     "peer1", password_digest1_w, random_timestamp, peer1_key_pair.getPublic());
    runExceptionTest("Known user alias with null public_key", key_store1,
                     "initializeAccount", "java.security.KeyStoreException", true,
                     "peer1", password_digest1, random_timestamp, null);
    runExceptionTest("Null password", key_store1,
                     "initializeAccount", "java.lang.NullPointerException", true,
                     "peer1", null, random_timestamp, peer1_key_pair.getPublic());

    HummerCertificate peer1_recieved_certificate = 
             (HummerCertificate)(runExceptionTest("Valid parameters (alias-\"user1\", peer1_key_pair.PublicKey)",
                                                   key_store1, "initializeAccount", null, false,
                                                   "peer1", password_digest1, random_timestamp,
                                                   peer1_key_pair.getPublic())[2]);
    runSimpleTest("peer1 (above initialized account alias)",  "No longer considered a \"new\" account",
                  !key_store1.isNewAccount("peer1"));
    runSimpleTest("Above returned HummerCertificate", "HummerCertificate is not null",
                  user1_recieved_certificate != null);
    runExceptionTest("Alias of account already initialized",
                     key_store1, "initializeAccount", "java.security.KeyStoreException", true,
                     "peer1", password_digest1,
                     random_timestamp, peer1_key_pair.getPublic());

    if(peer1_recieved_certificate != null) {
      runSimpleTest("PublicKey in certificate equals PublicKey of \"peer1\"", "true", 
                   peer1_recieved_certificate.getPublicKey().equals(peer1_key_pair.getPublic()));
      runSimpleTest("HummerCertificate.isExpired for newly created HummerCertificate",
                    "false",!peer1_recieved_certificate.isExpired());
      runSimpleTest("HummerCertificate.isValid for newly created HummerCertificate", "true",
                    peer1_recieved_certificate.isValid(key_store1.getPersonalVerifySignature()));
    }
    caught_exception = !((Boolean)(runExceptionTest("Did not throw exception when " +
                                         "retrieving key (peer)",
                                         key_store1, "getKey",
                                         "java.security.KeyStoreException", false, 
                                         new Integer(AliasInfo._PUBLIC_KEY_TYPE), "peer1")[0])).booleanValue();
    if(!caught_exception) {
      runSimpleTest("key_store1.getPublicKey(\"peer1\")", "PublicKey equal to peer1 key pair",
                    key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "peer1").equals(peer1_key_pair.getPublic()));
    }
    passed = false;
		/*
		The following has not been tested yet, so exceptions should be trapped just in case
		*/
    try {
      passed = key_store1.getPersonalKeyPair("peer1").equals(key_store1.getPersonalKeyPair());
    }
    catch(Exception e) {}
    runSimpleTest("Compare getPersonalKeyPair with \"peer1\" " +
                  "with current key_store KeyPair", "true", passed);



	/*
	Test initialization for server
	*/
    note("Server Account (server2)");
    byte[] server2_digest = StaticMethods.getDigest(server2_pass.getBytes(),
                                                    random_timestamp, server2_key_pair.getPublic());
    HummerCertificate server_recieved_certificate = 
             (HummerCertificate)(runExceptionTest("Valid parameters (alias-\"server2\", server2_key_pair.PublicKey)",
                                                   key_store1, "initializeAccount", null, false,
                                                   "server2", server2_digest, random_timestamp,
                                                   server2_key_pair.getPublic())[2]);
    runSimpleTest("server2 (above initialized account alias)",  "No longer considered a \"new\" account",
                  !key_store1.isNewAccount("server2"));
    runSimpleTest("Above returned HummerCertificate", "HummerCertificate is not null",
                  server_recieved_certificate != null);
    if(server_recieved_certificate != null) {
      runSimpleTest("PublicKey in certificate equals PublicKey of \"server2\"", "true",
                    server_recieved_certificate.getPublicKey().equals(server2_key_pair.getPublic()));
      runSimpleTest("HummerCertificate.isExpired method for new HummerCertificate", "false",
                    !server_recieved_certificate.isExpired());
      runSimpleTest("HummerCertificate.isValid for new HummerCertificate", "true",
                    server_recieved_certificate.isValid(key_store1.getPersonalVerifySignature()));
    }
    caught_exception = !((Boolean)(runExceptionTest("Did not catch exception when retrieving " +
                                                  "key (server2)",
                                                  key_store1, "getKey",
                                                  null, false, new Integer(AliasInfo._PUBLIC_KEY_TYPE),
                                                  "server2")[0])).booleanValue();
    if(!caught_exception) {
      runSimpleTest("getKey method with \"server2\" equals PublicKey for \"server2\" " +
                    "(the account that was initialized)", "true",
                    key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "server2").equals(server2_key_pair.getPublic()));
    }
    runSimpleTest("equalAccessTable for \"server2\" with access table given to \"server2\" originally", "true",
                 key_store1.equalAccessTable("server2", access_table2));
    passed = false;
		/*
		The following has not been tested yet, so exceptions should be trapped just in case
		*/
    try {
      passed = key_store1.getPersonalKeyPair("server2").equals(key_store1.getPersonalKeyPair());
    }
    catch(Exception e) {}
    runSimpleTest("Compare getPersonalKeyPair with \"server2\" with current key_store KeyPair", "true", passed);


		/* ***********************
		resetAlias testing
		*************************/
    newMethod("resetAlias");
    key_store1.resetAlias("server2", "new_password".getBytes());
    runSimpleTest("Initiailized account alias (server2)",
                  "Is no longer a new account (isNewAccount method=false)",
                  key_store1.isNewAccount("server2"));
    server2_digest = StaticMethods.getDigest("new_password".getBytes(),
                                             random_timestamp,
                                             server2_key_pair.getPublic());
    runExceptionTest("Re-initializing account (server2)", key_store1, "initializeAccount", null, false,
                     "server2", server2_digest, random_timestamp, server2_key_pair.getPublic());
    key_store1.resetAlias("user2", "new_password_again".getBytes());
    byte[] user2_digest = StaticMethods.getDigest("new_password_again".getBytes(),
                                                  random_timestamp,
                                                  user2_key_pair.getPublic());
    runExceptionTest("Initializing an account that was reset (new password) before ever being initialized",
                     key_store1, "initializeAccount", null, false,
                     "user2", user2_digest, random_timestamp, user2_key_pair.getPublic());
    key_store1.resetAlias("user2", user2_pass.getBytes());


		/* ***********************
		generateHummerCertificate testing
		*************************/
    newMethod("generateHummerCertificate");
    runExceptionTest("Unitiailized account alias (user2)", key_store1, "generateHummerCertificate",
                     null, true, "user2");
    HummerCertificate certificate_generated = 
                     (HummerCertificate)(runExceptionTest("Initialized account alias (user1)", key_store1,
                                          "generateHummerCertificate", null, false, "user1")[2]);
    if(certificate_generated != null) {
      runSimpleTest("Generated certificate expiration date", "Equals key store's KeyPair expriation date",
                    key_store1.getPersonalKeyPairExpiration().equals(certificate_generated.getExpirationDate()));

      runSimpleTest("Generated certificate access table", "Equals account's access table",
                    key_store1.equalAccessTable("user1", certificate_generated.getAccessTable()));

      runSimpleTest("Generated certificate PublicKey", "Equals account's PublicKey",
                    key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "user1").equals(certificate_generated.getPublicKey()));
      runSimpleTest("Generated certificate is valid for PublicKey associated with given alias (user1)",
                    "boolean true",
                    certificate_generated.isValid(key_store1.getPersonalVerifySignature("user1")));
    }

		/* ***********************
		getKey testing
		*************************/
    newMethod("getKey");
    runSimpleTest("User alias (user1)", "PublicKey equal to PublicKey for alias KeyPair",
                  user1_key_pair.getPublic().equals(key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "user1")));
    runSimpleTest("Server alias (server2)", "PublicKey equal to PublicKey for alias KeyPair",
                  server2_key_pair.getPublic().equals(key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "server2")));


		/* ***********************
		setKey testing
		*************************/
    PublicKey user1_old_public_key = user1_key_pair.getPublic(),
              server2_old_public_key = server2_key_pair.getPublic();
    user1_key_pair = StaticMethods.generateKeyPair();
    server2_key_pair = StaticMethods.generateKeyPair();
    runExceptionTest("Unititialized account alias (user2)", key_store1, "setKey", null,
                     true,  new Integer(AliasInfo._PUBLIC_KEY_TYPE), "user2",
                     user2_key_pair.getPublic());
    key_store1.setKey(AliasInfo._PUBLIC_KEY_TYPE, "user1", user1_key_pair.getPublic());
    key_store1.setKey(AliasInfo._PUBLIC_KEY_TYPE, "server2", server2_key_pair.getPublic());
    runSimpleTest("User alias (user1) for getKey", "Equal PublicKey for new user1 PublicKey",
                  key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "user1").equals(user1_key_pair.getPublic()));
    runSimpleTest("Server alias (server2) for getKey", "Equal PublicKey for server2 PublicKey",
                  key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "server2").equals(server2_key_pair.getPublic()));


		/* ***********************
		isNewAccount testing
		*************************/
    newMethod("isNewAccount");
    runSimpleTest("Uninitialized account alias (user2)", "true", key_store1.isNewAccount("user2"));
    runSimpleTest("Initialized account alias (user1)", "false", !key_store1.isNewAccount("user1"));

		/* ***********************
		getDigest testing
		*************************/
    newMethod("getDigest");
    runExceptionTest("Initialized account alias (user1)", key_store1, "getDigest",
                     null, true, "user1", random_timestamp, null);
    runExceptionTest("Null PublicKey (user2 - uninitialized account)", key_store1, "getDigest",
                     null, true, "user2", random_timestamp, null);
    Object[] exception_test_return = runExceptionTest("alias is \"user2\"  PublicKey for server",
                                     key_store1, "getDigest", null, false,
                                     "user2", random_timestamp, server1_key_pair.getPublic());
    byte[] message_digest = (byte[])(exception_test_return[2]);
    if(message_digest != null) {
      runSimpleTest("message digest bytes returned by getDigest",
                    "Equal to message digest returnd by StaticMethods.getDigest with equivalent values",
                    MessageDigest.isEqual(message_digest,
                                          StaticMethods.getDigest(user2_pass.getBytes(),
                                                                  random_timestamp,
                                                                  server1_key_pair.getPublic())));
    }
    else {
      runSimpleTest("message digest bytes returned by getDigest", "non-null value", false);
    }

		/* ***********************
		setHummerCertificate (beginning) testing
		*************************/
    newMethod("setHummerCertificate (partial)");
    HummerCertificate server1_certificate_valid,
                      user1_certificate_valid,
                      server2_certificate_valid,
                      server2_certificate_wrong_public_key,
                      server2_certificate_invalid,
                      server2_certificate_expired;
    Signature server1_sign = StaticMethods.getSigningSignatureObject(server1_key_pair.getPrivate()),
              server1_verify = StaticMethods.getVerifyingSignatureObject(server1_key_pair.getPublic()),
              server2_sign = StaticMethods.getSigningSignatureObject(server2_key_pair.getPrivate()),
              server2_verify = StaticMethods.getVerifyingSignatureObject((PublicKey)key_store1.getKey(AliasInfo._PUBLIC_KEY_TYPE, "server2")),
              user1_sign = StaticMethods.getSigningSignatureObject(user1_key_pair.getPrivate());

    server2_certificate_valid = new HummerCertificate(server2_sign,
                                                      key_store1.getPersonalKeyPair().getPublic(),
                                                      null,
                                                      key_store1.getPersonalKeyPairExpiration());
    server2_certificate_wrong_public_key = new HummerCertificate(server2_sign,
                                                      server1_key_pair.getPublic(),
                                                      null,
                                                      key_store1.getPersonalKeyPairExpiration());
    server2_certificate_invalid = new HummerCertificate(server1_sign,
                                                      key_store1.getPersonalKeyPair().getPublic(),
                                                      null,
                                                      key_store1.getPersonalKeyPairExpiration());
    server2_certificate_expired = new HummerCertificate(server2_sign,
                                                      key_store1.getPersonalKeyPair().getPublic(),
                                                      null,
                                                      new GregorianCalendar());
    server1_certificate_valid = new HummerCertificate(server1_sign,
                                                      key_store1.getPersonalKeyPair().getPublic(),
                                                      null,
                                                      key_store1.getPersonalKeyPairExpiration());
    user1_certificate_valid = new HummerCertificate(user1_sign,
                                                    key_store1.getPersonalKeyPair().getPublic(),
                                                    null,
                                                    key_store1.getPersonalKeyPairExpiration());

    runExceptionTest("Unitialized account (server1)", key_store1, "setHummerCertificate",
                     "java.security.KeyStoreException", true, "server1", server1_certificate_valid);
    runExceptionTest("User account (user1)", key_store1, "setHummerCertificate",
                    "java.security.KeyStoreException", true, "user1", user1_certificate_valid);
    runExceptionTest("Initialized account with wrong PublicKey contained in HummerCertificate",
                     key_store1, "setHummerCertificate", "java.security.KeyStoreException", true,
                     "server2", server2_certificate_wrong_public_key);
    runExceptionTest("Initialized server account with HummerCertificate signed by wrong PublicKey (server2)",
                     key_store1, "setHummerCertificate", "java.security.KeyStoreException", true,
                     "server2", server2_certificate_invalid);
    runExceptionTest("Initialized server account with expired HummerCertificate (server2)",
                     key_store1, "setHummerCertificate", "java.security.KeyStoreException", true,
                     "server2", server2_certificate_expired);

		/* ***********************
		getHummerCertificate testing
		*************************/
    newMethod("getHummerCertificate (partial)");
    runExceptionTest("Uninitialized account (server1)", key_store1, "getHummerCertificate",
                    "java.security.KeyStoreException", true,
                    "server1");
    runExceptionTest("User account (user1)", key_store1, "getHummerCertificate",
                    "java.security.KeyStoreException", true,
                    "user1");
    runExceptionTest("Getting HummerCertificate for server account without " +
                     "setting HummerCertificate (server2)",
                     key_store1, "getHummerCertificate", "java.security.KeyStoreException", true,
                     "server2");

		/* **********************
		setHummerCertificate (completed)
		************************/
    newMethod("setHummerCertificate (completed)");
    runExceptionTest("Initialized server account and valid HummerCertificate", key_store1, "setHummerCertificate",
                     null, false, "server2", server2_certificate_valid);

    newMethod("getHummerCertificate (completed)");
    Object[] exception_return = runExceptionTest("Initialized server account (server2)",
                                                 key_store1, "getHummerCertificate",
                                                 null, false, "server2");
    runSimpleTest("Intialized server account alias (server2)", "Non-null HummerCertificate",
                  exception_return[2] != null);

		/* ***********************
		isCertificateHonored testing
		*************************/
    newMethod("isCertificateHonored");
    runSimpleTest("User alias (user1)", "false", !key_store1.isCertificateHonored("user1"));
    runSimpleTest("Non-trusted server (server1)", "false", !key_store1.isCertificateHonored("server1"));
    runSimpleTest("Trusted server (server2)", "true", key_store1.isCertificateHonored("server2"));

		/* ***********************
		Partial testing of getPersonalKeyPair(int,Object)
		*************************/
    newMethod("getPersonalKeyPair (int,Object) (partial)");
    int timestamp_index = HummerKeyStore._BACKUP_TIMESTAMP_INDEX,
        public_key_index = HummerKeyStore._BACKUP_PUBLIC_KEY_INDEX;
    runSimpleTest("Null PublicKey", "null", key_store1.getPersonalKeyPair(public_key_index, null) == null);
    runSimpleTest("Unknown PublicKey", "null",
                 key_store1.getPersonalKeyPair(public_key_index, user2_key_pair.getPublic()) == null);
    KeyPair result_key_pair = key_store1.getPersonalKeyPair(public_key_index,
                                            key_store1.getPersonalKeyPair().getPublic());
    runSimpleTest("PublicKey for HummerKeyStore current KeyPair",
                  "non-null PublicKey", result_key_pair != null);
    if(result_key_pair != null) {
      runSimpleTest("PublicKey for HummerKeyStore current KeyPair",
                    "PublicKey equal to current PublicKey for HummerKeyStore",
                    result_key_pair.equals(key_store1.getPersonalKeyPair()));
    }

		/* ***********************
		Partial testing of getPersonalKeyPair(String)
		*************************/
    newMethod("getPersonalKeyPair (int,Object) (partial)");
    runExceptionTest("Unknown alias (unknown)", key_store1, "getPersonalKeyPair",
                     null, true, "unknown");
    exception_return = runExceptionTest("Uninitialized account (user2)",
                                        key_store1, "getPersonalKeyPair",
                                        null, false, "user2");
    if(((Boolean)exception_return[0]).booleanValue()) {
      runSimpleTest("Uninitialized account (user2)", "Null KeyPair object", exception_return[2] == null);
    }
    runSimpleTest("Alias for initialized account (user1)", "Current KeyPair for HummerKeyStore",
                 key_store1.getPersonalKeyPair("user1").equals(
                               key_store1.getPersonalKeyPair()));


		/* ***********************
		updateKeyPair testing
		*************************/
    newMethod("updateKeyPair");
    GregorianCalendar key_store_expiration_date1 = key_store1.getPersonalKeyPairExpiration();
    KeyPair old_key_pair = key_store1.getPersonalKeyPair();
    Signature old_verify = key_store1.getPersonalVerifySignature();
    HummerCertificate old_hummer_certificate = key_store1.getPersonalHummerCertificate();
    runSimpleTest("false (update not forced)", "false (KeyPair has not been changed)",
                  !key_store1.updateKeyPair(false));
    runSimpleTest("Expiration date after running updateKeyPair method (KeyPair has not changed)",
                  "Equal to previous expiration date",
                  key_store_expiration_date1.equals(key_store1.getPersonalKeyPairExpiration()));
    runSimpleTest("KeyPair after calling updateKeyPair (KeyPair has not changed)", "Equal to previous KeyPair",
                  old_key_pair.equals(key_store1.getPersonalKeyPair()));
    runSimpleTest("HummerCertificate after calling updateKeyPair (KeyPair has not changed)",
                  "Equal to previous HummerCertificate",
                  old_hummer_certificate.equals(key_store1.getPersonalHummerCertificate()));

    runSimpleTest("true (update is forced)", "true (KeyPair has been changed)",
                  key_store1.updateKeyPair(true));
    runSimpleTest("Expiration date after running updateKeyPair (KeyPair has changed)",
                  "Not equal to previous expiration date",
                  !key_store_expiration_date1.equals(key_store1.getPersonalKeyPairExpiration()));
    runSimpleTest("KeyPair after calling updateKeyPair (KeyPair has changed)",
                  "Not equal to previous KeyPair",
                  !old_key_pair.equals(key_store1.getPersonalKeyPair()));
    runSimpleTest("HummerCertificate after calling updateKeyPair (KeyPair has changed)",
                  "Not equal to previous HummerCertificate",
                  !old_hummer_certificate.equals(key_store1.getPersonalHummerCertificate(), old_verify));
    KeyPair new_key_pair = key_store1.getPersonalKeyPair();
    HummerCertificate new_certificate = key_store1.getPersonalHummerCertificate("user1");
    runSimpleTest("Certificate associated with alias (user1)",
                  "Is valid with PublicKey associated with alias (user1)",
                  new_certificate.isValid(key_store1.getPersonalVerifySignature("user1")));
    runSimpleTest("Certificate associated with alias (user1)",
                  "PublicKey contained in the HummerCertificate equals current PublicKey",
                  Arrays.equals(new_certificate.getPublicKey().getEncoded(),
                                key_store1.getPersonalKeyPair().getPublic().getEncoded()));

		/* ************************
		getPersonalHummerCertificate testing
		**************************/
   newMethod("getPersonalHummerCertificate");
   runExceptionTest("Initialized account alias (user2)", key_store1,
                    "getPersonalHummerCertificate", null, true, "user2");
   HummerCertificate certificate = (HummerCertificate)(runExceptionTest("Initialized account (user1)",
                                                          key_store1, "getPersonalHummerCertificate",
                                                          null, false, "user1")[2]);

   if(runSimpleTest("Certificate recieved for alias (user2)", "Is not null", certificate != null)) {
     runSimpleTest("Certificate recieved for alias (user2)",
                   "Is valid using PublicKey associated with alias",
                   certificate.isValid(key_store1.getPersonalVerifySignature("user1")));
   }


		/* ***********************
		getPersonalKeyPair(String) testing (completed)
		*************************/
    newMethod("getPersonalKeyPair(String) (completed)");
    runExceptionTest("Unknown alias (unknown)", key_store1, "getPersonalKeyPair",
                     "edu.uidaho.hummer.util.UnknownAliasException", true, "unknown");
    exception_return = runExceptionTest("Uninitialized account (user2)",
                                        key_store1, "getPersonalKeyPair",
                                        null, false, "user2");
    if(((Boolean)exception_return[0]).booleanValue()) {
      runSimpleTest("Uninitialized account (user2)", "Null KeyPair object", exception_return[2] == null);
    }
    KeyPair current_key_pair = key_store1.getPersonalKeyPair("user1");
    runSimpleTest("Initialized account alias (user1)",
                  "PublicKey equal to PublicKey before updateKeyPair method called",
                 current_key_pair.getPublic().equals(old_key_pair.getPublic()));
    runSimpleTest("Initialized account alias (user1)",
                  "PrivatecKey equal to PrivatecKey before updateKeyPair method called",
                 current_key_pair.getPrivate().equals(old_key_pair.getPrivate()));



		/* ***********************
		getPersonalKeyPair(int, Object) testing (completed)
		*************************/
    newMethod("getPersonalKeyPair(int,Object) (completed)");
    runSimpleTest("Null PublicKey", "null", key_store1.getPersonalKeyPair(public_key_index, null) == null);
    runSimpleTest("PublicKey that was never one of the HummerKeyStore's PublicKeys", "null",
                 key_store1.getPersonalKeyPair(public_key_index, server1_key_pair.getPublic()) == null);
    result_key_pair = key_store1.getPersonalKeyPair(public_key_index, old_key_pair.getPublic());
    runSimpleTest("PublicKey from KeyPair before updateKeyPair method called", "non-null value", result_key_pair != null);
    if(result_key_pair != null) {
      runSimpleTest("Compare KeyPair returned from above", "Equal to known old KeyPair",
                    result_key_pair.getPublic().equals(old_key_pair.getPublic()) &&
                    result_key_pair.getPrivate().equals(old_key_pair.getPrivate()));
    }

		/* ***********************
		getAliases Testing
		*************************/
    newMethod("getAliases");
    Vector aliases = new Vector();
    aliases.add("admin");
    aliases.add("server1");
    aliases.add("server2");
    aliases.add("user1");
    aliases.add("user2");
    aliases.add("peer1");
    aliases.add("peer2");
    key_store1.removeAlias("Empty User1");
    key_store1.removeAlias("Empty User2");
    key_store1.removeAlias("Empty Peer1");
    key_store1.removeAlias("server_empty1");
    key_store1.removeAlias("server_empty2");
    Vector alias_list = new Vector();
    for(Enumeration enum = key_store1.getAliases(); enum.hasMoreElements();) {
      alias_list.add(enum.nextElement());
    }
    runSimpleTest("Vector of aliases in key store", "Size equals " + aliases.size(),
                  alias_list.size() == aliases.size());
    boolean all_contained = true;
    for(Enumeration enum = aliases.elements(); all_contained && enum.hasMoreElements();) {
      all_contained = alias_list.contains(enum.nextElement());
    }
    runSimpleTest("Vector of aliases in key store", "Elements contained are equal",
                  all_contained);

		/* ***********************
		Store/Load Testing
		*************************/
    newMethod("store/load");
    KeyPair write_key_pair = key_store1.getPersonalKeyPair(), read_key_pair;
    Exception e = (Exception)(runExceptionTest("Storing server key store to local machine address (true, null)",
                              key_store1, "store", null, false, Boolean.TRUE, null)[1]);
    key_store2 = (DefaultKeyStore)HummerKeyStore.getInstance();
    e = (Exception)(runExceptionTest("Loading server key store from local machine address (true, null)",
                                     key_store2, "load", null, false, Boolean.TRUE, null)[1]);
    if(e != null) {
      errorPrintln("Caught: ");
      e.printStackTrace();
    }
    read_key_pair = key_store2.getPersonalKeyPair();
    runSimpleTest("key_store2.size() method called (object read from file)",
                  "Value returned by key_store1.size()",
                  key_store1.size() == key_store2.size());
    runSimpleTest("HummerKeyStore read from file contains admin alias", "true",
                  key_store2.contains("admin"));
    runSimpleTest("HummerKeyStore read from file contains user1 alias", "true",
                  key_store2.contains("user1"));
    runSimpleTest("HummerKeyStore read from file contains user2 alias", "true",
                  key_store2.contains("user2"));
    runSimpleTest("HummerKeyStore read from file contains server1 alias", "true",
                  key_store2.contains("server1"));
    runSimpleTest("HummerKeyStore read from file contains server2 alias", "true",
                  key_store2.contains("server2"));
    if(runSimpleTest("HummerKeyStore read from file contains non-null HummerCertificate", "true",
                     (key_store2.getPersonalHummerCertificate() != null))) {
      runSimpleTest("HummerKeyStore read from file contains valid HummerCertificate " +
                    "with PublicKey from KeyPair", "true",
                    key_store2.getPersonalHummerCertificate().isValid(
                               key_store2.getPersonalVerifySignature()));
    }

		/*
		For some reason, the two key pairs cannot be simply compared, but rather the public and
		private keys must be compared (there may be some sort of date created involved causing this)
		*/
    runSimpleTest("Compare PublicKey and PrivateKey's in HummerKeyStore written equal the keys " +
                  "in the HummerKeyStore read from file", "equal=true",
                  write_key_pair.getPublic().equals(read_key_pair.getPublic()) &&
                  write_key_pair.getPrivate().equals(read_key_pair.getPrivate()));
    runSimpleTest("Compare KeyPair expiration dates for HummerKeyStore written and HummerKeyStore read from file",
                  "equal=true",
                  key_store1.getPersonalKeyPairExpiration().equals(key_store2.getPersonalKeyPairExpiration()));
      PublicKey[] public_keys1 = null, public_keys2 = null;
      PrivateKey[] private_keys1 = null, private_keys2 = null;
      if(runSimpleTest("Compare number of backup keys for HummerKeyStore written and HummerKeyStore read from file", 
             "equal=true",
             key_store1.getPersonalBackupKeys()[0].size() == key_store2.getPersonalBackupKeys()[0].size())) {
        Object[] backup_key_pair1 = (key_store1.getPersonalBackupKeys()[0]).toArray(),
                  backup_key_pair2 = (key_store2.getPersonalBackupKeys()[0]).toArray();
        runSimpleTest("Compare loaded HummerKeyStore written with those in HummerKeyStore read from file",
                      "equal=true",
                      Arrays.equals(backup_key_pair1, backup_key_pair2) && Arrays.equals(private_keys1, private_keys2));
      }
  }
}