/*
Java Hummer Project
File: InitializeTables.java
Date: 12-22-98
Author: Tiffany Cavanaugh
Description: 
             
 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.tool;

import java.net.*;
import java.sql.*;
import java.io.*;
import java.util.*;

import edu.uidaho.hummer.util.StaticMethods;
  
/**
This class takes care of the initialization of the database,
<h4>Class Usage and Flow:</h4>
<ul>
<li>Uses the _constants object to retrieve system information from the properties file.
<li>Obtains a connection to the database using _constants object.
</ul>
@author Tiffany Cavanaugh
*/
public class InitializeTables{
  /**
boolean value to determine whether or not a connection to the database was successful.
  */
  public boolean _is_connected = false;
  /**
BufferedReader stores the data from the file used to initialize the database.
  */
  public BufferedReader _buffer = null;
  /**
String representing the machine that is accessing the database.
  */
  public String sql_user = new String();
  /** 
String representing the url where the database is located.
  */
  public String sql_url= new String();
  /**
String that holds the password to the database.
  */
  public String sql_driver = new String(); 
  public String sql_password = new String(); 
  public Connection connection = null;
  public Statement statement = null;
  public Vector test = new Vector(); 
  public Vector _drop_tables = new Vector();
  /**
     The constructor for InitializeTables that creates the connection with the database.
  */
  public InitializeTables() {

    Properties _constants = new Properties(System.getProperties());
    FileInputStream _file = null;
    
    
    try {
      _constants = StaticMethods.getHummerProperties();
    }
    catch (FileNotFoundException fnfe) {
      System.err.println("InitializeTables: Hummer.properties file not found.");
      System.exit(0);
    }
    catch (IOException ioe) {
      System.err.println("InitializeTables: File Hummer.properties not valid.");
      System.exit(0);
    }
    catch (NumberFormatException ne) {
      System.err.println("InitializeTables: Bad port.MESSAGE_PORT property.");
      System.exit(0);
    }  
    
    
    try {
      String address = InetAddress.getLocalHost().getHostAddress();
      StringTokenizer nametoken = new StringTokenizer(InetAddress.getLocalHost().getHostName(), ".");
      if (_constants.containsKey("jdbc.url." + address)) {
	sql_url = _constants.getProperty("jdbc.url." + address)
	  + nametoken.nextToken();
      }
      else {
	sql_url = _constants.getProperty("jdbc.url") 
	  + nametoken.nextToken();
      }      
      if (_constants.containsKey("jdbc.driver." + address)) {
	sql_driver = _constants.getProperty("jdbc.driver." + address);
      }
      else {
	sql_driver = _constants.getProperty("jdbc.driver");
      }

      if (_constants.containsKey("jdbc.user." + address)) {
	sql_user = _constants.getProperty("jdbc.user." + address);
      }
      else {
	sql_user = _constants.getProperty("jdbc.user");
      }
      if (_constants.containsKey("jdbc.password." + address)) {
	sql_password = _constants.getProperty("jdbc.password." + address);
      }
      else {
	sql_password = _constants.getProperty("jdbc.password");
      }		

      Class.forName(sql_driver);       
      _is_connected = true;
    }
    catch (UnknownHostException uhe) {
      System.err.println("InitializeTables:" + uhe);
    }
    catch (ClassNotFoundException cnfe) {
      System.err.println("InitializeTables:" + cnfe);
      System.exit(0);
    }      
  }
  
  /**
     Main: makes the calls to each method.
  */
  public static void main(String args[]) throws SQLException {
   
  
   String user_dir = new String();
   String file_separator = new String();
   InitializeTables initial= new InitializeTables(); 
   
    if(args.length > 0){
      initial.openFile(args[0]);

      initial.sqlStatements();
      System.out.println("hello");
      initial.dropDatabase();
      initial.createDatabase();
      initial.addElementsToDatabase();
    }
    else{
      Properties _constants = new Properties(System.getProperties());
      FileInputStream _file = null;
     
      
      try {
	_constants  = StaticMethods.getHummerProperties();
        if(_file != null) {
          _file.close();
        }
      }
      catch (FileNotFoundException fnfe) {
	System.err.println("InitializeTables: Hummer.properties file not found.");
	System.exit(0);
      }
      catch (IOException ioe) {
	System.err.println("InitializeTables: File Hummer.properties not valid.");
	System.exit(0);
      }
      catch (NumberFormatException ne) {
	System.err.println("InitializeTables: Bad port.MESSAGE_PORT property.");
	System.exit(0);
      }  
      if(_constants.containsKey("jdbc.tools.add_values")){
      String openfile = new String(user_dir +file_separator 
				   +_constants.getProperty
				   ("jdbc.tools.add_values"));
      initial.openFile(openfile);
      initial.sqlStatements();
      initial.dropDatabase();
      initial.createDatabase();
      initial.addElementsToDatabase();
      }
    }
    
  }//end main
  
  
  /**
    
  <h4>Method Usage and Flow:</h4>
  <ul>
  <li>sqlStatements method places the appropriate sql queries into a vector, and sends them to the database.
  </ul>
  */
  public void sqlStatements() throws SQLException {
    if (_is_connected) {
        
	try{
	  connection = DriverManager.getConnection
	    (sql_url, sql_user, sql_password); 
	  System.out.println("Connection successful");
	  statement= connection.createStatement();
	}
	catch(SQLException e){
	  System.err.println("sqlStatements: " +e);
	}
	
    }
    else{
      System.err.println("no connection was made");
    }
  }
  /**
This method will drop all existing tables in the database without destroying the database.
<h4>Method Usage and Flow:</h4>
<ul>
<li>Places all the appropriate SQL queries in the vector _drop_tables.
</ul>
  */

  public void dropDatabase(){
    
    /******************************************/
    //sql statements that drop tables
    
    String S = new String();
    
    S = "drop table LOG;";
    _drop_tables.addElement(S);
    S = "drop table FILTER;";
    _drop_tables.addElement(S);
    S = "drop table ACCEPT_LIST;";
    _drop_tables.addElement(S);
    S = "drop table FORWARD_LIST;";
    _drop_tables.addElement(S);
    S = "drop table FORWARD_PEER;";
    _drop_tables.addElement(S);
    S = "drop table MANAGER;";
    _drop_tables.addElement(S);
    S = "drop table levels;";
    _drop_tables.addElement(S);
    S = "drop table config_time;";
    _drop_tables.addElement(S);
    S = "drop table subordinates;";
    _drop_tables.addElement(S);
    S = "drop table pid;";
    _drop_tables.addElement(S);
    S = "drop table kill_file;";
    _drop_tables.addElement(S);
    S = "drop table pgroups;";
    _drop_tables.addElement(S);
    S = "drop table peers;";
    _drop_tables.addElement(S);
    S = "drop table message;";
    _drop_tables.addElement(S);
  }
  
  /**
This method will create all tables needed in to create the database.
 <h4>Method Usage and Flow:</h4>
 <ul>
 <li>Places all the appropriate SQL queries for creating tables into the vector test.
</ul>
  */
  public void createDatabase(){
    /*****************************************/   
    //sql statements that recreate the tables
    String S = new String();
    try{
      S = "CREATE TABLE LOG (" +
	"trust int,"+	
	"coop int," +
	"integ int," +
	"ldate date,"+
	"ltime date," +
	"description text," +
	"hostip text," +
	"file text," +
	"tool text);";
      test.addElement(S);
      
      S = "CREATE TABLE FILTER (" +
	"expression text,"+
	"date_start date,"+
	"date_end date,"+
	"time_start text,"+
	"time_end text,"+
	"trust int,"+
	"coop int,"+
	"integ int,"+
	"log int,"+
	"alert int,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE ACCEPT_LIST ("+
	"filter text,"+
	"start_addr text,"+
	"end_addr text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE FORWARD_LIST ("+
	"filter text,"+
	"host text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE FORWARD_PEER ("+
	"filter text,"+
	"pgroup text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE MANAGER("+
	"manager text);";
      test.addElement(S);
      
      S = "CREATE TABLE LEVELS ("+
	"start_addr text,"+
	"end_addr text,"+
	"trust int,"+
	"coop int,"+
	"integ int,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE CONFIG_TIME ("+
	"ctime int);";
      test.addElement(S);
      
      S = "CREATE TABLE SUBORDINATES ("+
	"hum_name text,"+
	"subord text);";
      test.addElement(S);
      
      S = "CREATE TABLE PID ("+
	"name text,"+
	"pid int);";
      test.addElement(S);
      
      S = "CREATE TABLE KILL_FILE ("+
	"start_addr text,"+
	"end_addr text,"+
	"inh int);";
      test.addElement(S);
      
      S = "CREATE TABLE PGROUPS ("+
	"pgroup text,"+
	"mod_addr text,"+
	"email text);";
      test.addElement(S);
      
      S = "CREATE TABLE PEERS("+
	"pgroup text,"+
	"mem_addr text,"+
	"peer_sub int,"+
	"peer_mod int);";
      test.addElement(S);
      
      S = "CREATE TABLE MESSAGE ("+
	"pgroup text,"+
	"mem_addr text,"+
	"action text,"+
	"rdate date,"+
	"rtime text,"+
	"status int,"+
	"sdate date,"+
	"stime text,"+
	"mpublic int);";
      test.addElement(S);
    }
    catch(Exception e){
      System.err.println("InitializeTables: sqlStatements: " +e);
    }
  }
  
  /**
This method adds the elements from the file to the vector test.
<h4>Method Usage and Flow:</h4>
<ul>
<li> A file that contains the initial data for each table is read and added to the vector.
<li>Creates an enumeration of each vector and sends the contents to the database.
</ul>
  */
  public void addElementsToDatabase(){
    String _this_line = new String();
    try{
      while (( _this_line = _buffer.readLine()) != null){
	test.addElement(_this_line);
      }
    }
    catch(IOException e){
      System.err.println("addElementsToDatabase: " +e);
    }
    
    try{
      _buffer.close();
    }
    catch(IOException e){
      System.err.println("addElementsToDatabase: " +e);
    }
    
    /*****************************************************/
    try{
     statement= connection.createStatement();
    }
    catch(SQLException e){
      System.err.println("Statement: " +e);
    }



     //enum = to all elements in vector
    Enumeration drop = _drop_tables.elements();
    Enumeration enum = test.elements();
    
    String drop_value = new String();
    while( drop.hasMoreElements()){
      try{ 
	drop_value = (String)drop.nextElement();
	statement.executeUpdate(drop_value);
      }
      catch(SQLException e){
    }
    }    
    while( enum.hasMoreElements()) {
      try{
	//executes our sql statements in the vector
	statement.executeUpdate((String)enum.nextElement());
      }
      catch(SQLException e) { }
    }

    try{
      connection.close();
    }
    catch(SQLException e){
      System.err.println("CloseConnection:" + e);
    }
  }
  
  /**
     Opens the initializing file that is read in on the command line, and places its elements into the database.
  <h4>Method Usage and Flow:</h4>
  <ul>
  <li>Takes the file read from command line and reads it into a buffer.
  </ul>
@param args The file containing intializing values.
  */
  
  public void openFile(String args){
    
    FileReader _log = null;
    String _logfile = args;
   
    try{
      _log = new FileReader(_logfile);
      _buffer = new BufferedReader(_log);
    }
    catch(FileNotFoundException e){
      System.err.println("InitializeTables: openFile: " +e);
    }
  }
}


