/*
Java Hummer Project
File: SysTool.java
Date: 12-30-98
Author: Tiffany Cavanaugh
Description:   This tool is designed to parse the sys system tool and send all                relevant information to its hummer.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package edu.uidaho.hummer.tool.SunOS;

import edu.uidaho.hummer.client.MessageI;
import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.tool.HummerTool;

import java.io.*;
import java.util.*;
import edu.uidaho.hummer.util.HummerKeyStore;

/**
The SysTool class is a thread in the class ToolThreads. SysTool sends a message to MessageI every five minutes.
<h4>Class Usage and Flow:</h4>
<ul>
<li> Constructor creates the thread with the jdbc.tools.systool property from the _constants object.
<li>The start() methed is launched because it is a thread.
<li>Loops the thread on boolean variable going.
<ul>
<li>I gets the local system informatioin running df and calls MessageI which sendsthe message to the database running.
<li>The information is gathered only when new data is available.
<li>The thread is terminated when the MessagingServer is terminated.
</ul>
@author Tiffany Cavanaugh, Chad Ward
*/
public class SysTool extends HummerTool {
 
	/**
	An buffered reader that reads from the system log.  This
	read is gathered by starting a process that reads from the
	system log, and then getting the input stream for it.
	*/
  private BufferedReader _log_reader;


/**
Creates the thread SysTool as a part of the thread group created by ToolThreads.
@param str Identifier for the thread.
@param toolthreadgroup The thread group where the thread is located.
@param _constants A Hummer properties file being passed in which removes
  the extra construction time for Properties objects.
*/
  public SysTool(Properties constants, ThreadGroup thread_group, HummerKeyStore key_store){
    super("System", thread_group, key_store);
    _logfile = constants.getProperty("jdbc.tools.systool");
    Runtime log = Runtime.getRuntime();
    Process process = null;
    try{
      process = log.exec(_logfile);   
      _log_reader  = new BufferedReader(new InputStreamReader
                                                     (process.getInputStream()));
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("SysTool:constructor:  " + e);
    }
		/*
		Because this thread will pause on reading from the file the wait time
		should be set to 0.  This will insure that changes are logged
		as soon as possible.
		*/
    _wait_time = 0;
  }

	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Returns a human readable description of this tool.
	</ul>
	*/
  public String getDescription() {
    String description = "Returns new lines entered in the system log.";
    return description;
  }

  /**
<h4>Method Usage and Flow:</h4>
<ul>
<li> Parses the output of the log file to the desired tokens. 
<li> Creates the message to send to MessageI.
@param this_line The data is sent to stringManipulator() a line at a time.
  
*/
  public void gatherInformation() throws Exception {  
    StringTokenizer token = null;
    String month, dd, time, site, daemon, msg, delimitor = " ";
    Vector information = new Vector();
    String current_line = null;
    try {
      current_line = _log_reader.readLine();
    }
    catch(IOException e) {
      _status = "Error: " + e;
    	 System.err.println("SysTool:gatherInformation: " +
                         "Could not gather information from system log: " + e);
      return;
    }

    msg = new String();
    token  =  new StringTokenizer(current_line,delimitor);
    month  =  new String(token.nextToken());
    dd     =  new String(token.nextToken());
    time   =  new String(token.nextToken());
    site   =  new String(token.nextToken());
    daemon =  new String(token.nextToken());
    while(token.hasMoreTokens()){
      msg    = msg.concat(" " + token.nextToken());
    }
    
    if((msg.indexOf("ftp.login.from")>=0) ||
       (msg.indexOf("logged.out")>=0) ||
       (msg.indexOf("login.incorrect")>=0) ||
       (msg.indexOf("su")>=0)){
      
      information.addElement(daemon + msg);
      
    }   
    
    MessageI outputToMessageI = new MessageI(_key_store);
    Message param_data = new Message();
    Enumeration enum;

    enum = information.elements();
    while(enum.hasMoreElements()){
 
      String   doTool[] = {"m ", (String) enum.nextElement(),  
			           "h " ,_host_name,
	                    "f " ,_logfile,  "s " ,"SysTool"};
    
      outputToMessageI.parseParameters(doTool, param_data);
      outputToMessageI.validateParameters(param_data);
      outputToMessageI.openMessageConnection(_host_name);
      outputToMessageI.sendToHummer(param_data);
      outputToMessageI.closeMessageConnection();
    }
  }
}