/*
Java Hummer Project
File: UptimeTool.java
Date: 12-30-98
Author: Tiffany Cavanaugh
Description:   This tool is designed to parse the bdf system tool and send all                relevant information to its hummer.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.tool.SunOS;

import java.io.InputStreamReader;
import java.io.IOException;
import java.io.BufferedReader;
import java.util.StringTokenizer;
import java.util.Properties;
import java.util.Vector;

import edu.uidaho.hummer.dataobject.Message;
import edu.uidaho.hummer.client.MessageI;
import edu.uidaho.hummer.tool.HummerTool;
import edu.uidaho.hummer.util.HummerKeyStore;

/**
The UptimeTool class is a thread in the class ToolThreads. UptimeTool sends a message to MessageI every five minutes.
<h4>Class Usage and Flow:</h4>
<ul>
<li> Constructor creates the thread with the jdbc.tools.uptimetool property from the _constants object.
<li>The start() methed is launched because it is a thread.
<li>Loops the thread on boolean variables _exit_value and _exit_status.
<ul>
<li>I gets the local system informatioin running df and calls MessageI which sendsthe message to the database running.
<li>The thread is terminated when the MessagingServer is terminated.
</ul>
@author Tiffany Cavanaugh, Chad Ward
*/
public class UptimeTool extends HummerTool {

	/**
	Creates the thread UptimeTool as a part of the thread group created by ToolThreads.
	@param constants A Hummer properties file being passed in which removes
		the extra construction time for Properties objects.
	*/
  public UptimeTool(Properties constants, ThreadGroup thread_group, HummerKeyStore key_store){
    super("Uptime", thread_group, key_store);
    _logfile = constants.getProperty("jdbc.tools.uptimetool");
  }
 
	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Returns a human readable description of this tool.
	</ul>
	*/
  public String getDescription() {
    String description = "Returns the amount of time that the system has been operating.  " +
                         "This value is gathered from a the Unix uptime tool.";
    return description;
  }



	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>The log file output is parsed until the relevent data is gathered.
	  <li>Then it is sent as a message to MessageI.
	</ul>
	@param _logfile  The buffer where all the data collected from system call is stored.
	*/
  public void gatherInformation() throws Exception {
    BufferedReader output = null; 
    String this_line;
    StringTokenizer token = null;
    String timer, times, user, load, load1, load2,
           load3, delimitor = "/ +/", loads, msngr = null;
    int exit_status = 0, exit_value = 0;
    Vector information = new Vector();
    StringTokenizer loadstring = null;
    
    Runtime log = Runtime.getRuntime();
    Process process = null;

    try{
      process = log.exec(_logfile);   
      output  = new BufferedReader(new InputStreamReader
				   (process.getInputStream()));
      exit_status = process.waitFor();    
      process.destroy();
      exit_value = process.exitValue();      
      log.gc();
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("UptimeTool:gatherInformation: " + e);
    }
    catch(InterruptedException e){
      _status = "Error: " + e;
      System.err.println("UptimeTool:gatherInformation: " + e);
    }


    try{
      while((this_line = output.readLine()) != null ){
        token = new StringTokenizer(this_line,delimitor);

        timer  = new String(token.nextToken() +token.nextToken());
        times = new String(token.nextToken() +token.nextToken());
        user  = new String(token.nextToken() +token.nextToken());
        load  = new String(token.nextToken() +token.nextToken() +token.nextToken());
        load1 = new String(token.nextToken());
        load2 = new String(token.nextToken());
        load3 = new String(token.nextToken());

        loads = new String(load1.concat(load2 +load3));
        	
	loadstring  = new StringTokenizer(loads,",");
        load1 = new String(loadstring.nextToken());
        load2 = new String(loadstring.nextToken());
        load3 = new String(loadstring.nextToken());
	msngr = new String("LOAD: " +load1 +" "
			   +load2 +" " +load3);

      }
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("UptimeTool:gatherInformation: " +e );
    }   
    
    try{
      output.close();
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("UptimeTool:gatherInformation: " +e);
    }

    MessageI outputToMessageI = new MessageI(_key_store);
    Message param_data = new Message();

   
    String   doTool[] = {"m ", msngr,  
			 "h " ,_host_name,
			 "f " ,_logfile,  "s " ,"uptimetool"};
    outputToMessageI.parseParameters(doTool, param_data);
    outputToMessageI.validateParameters(param_data);
    outputToMessageI.openMessageConnection(_host_name);
    outputToMessageI.sendToHummer(param_data);
    outputToMessageI.closeMessageConnection();
  }
}