/*
Java Hummer Project
File: Hummer.java
Date: 12-30-98
Author: Tiffany Cavanaugh
Description:   This tool is designed to parse the who system tool and send all                relevant information to its hummer.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.tool.SunOS;

import edu.uidaho.hummer.client.MessageI;
import edu.uidaho.hummer.dataobject.Message;

import java.io.*;
import java.util.*;

import edu.uidaho.hummer.tool.HummerTool;
import edu.uidaho.hummer.util.HummerKeyStore;

/**
The WhoTool class is a thread in the class ToolThreads. WhoTool sends a message to MessageI every five minutes.
<h4>Class Usage and Flow:</h4>
<ul>
<li> Constructor creates the thread with the jdbc.tools.whotool property from the _constants object.
<li>The start() methed is launched because it is a thread.
<li>Loops the thread on boolean variables _exit_value and _exit_status.
<ul>
<li>I gets the local system informatioin running df and calls MessageI which sendsthe message to the database running.
<li>The thread is terminated when the MessagingServer is terminated.
</ul>

@author Tiffany Cavanaugh, Chad Ward
*/
public class WhoTool extends HummerTool {
  
 /**
Creates the thread WhoTool as a part of the thread group created by ToolThreads.
@param str Identifier for the thread.
@param ToolGroup The thread group where the thread is located.
@param _constants A Hummer properties file being passed in which removes
  the extra construction time for Properties objects.
*/ 
  public WhoTool(Properties constants, ThreadGroup thread_group, HummerKeyStore key_store){
    super("Who", thread_group, key_store);
    _logfile = constants.getProperty("jdbc.tools.whotool");
  }

	/**
	<h4>Method Usage and Flow:</h4>
	<ul>
	  <li>Returns a human readable description of this tool.
	</ul>
	*/
  public String getDescription() {
    String description = "Returns the names of the users on the system.  " +
                         "This value is gathered from a the Unix who tool.";
    return description;
  }

  /**
<h4>Method Usage and Flow:</h4>
<ul>
<li>This method is where the information from the logfile is gathered and parsed by implementing a separate process. 
<li> The data is put into a buffer in order to manipulate it accordingly.  
<li>The parsed message from is then sent to MessageI. 
</ul>
@param _logfile The buffer where all the data collected from system call is stored.
*/
  public void gatherInformation() throws Exception {
    String msngr = "";
    BufferedReader output = null;
    String delimitor = ""; 
    String this_line;
    StringTokenizer token = null;
    String name;
    Object users;
    String user;
    int exit_value = 0, exit_status = 0;
    Vector token_vector = new Vector(),
           information = new Vector();
    

    Runtime log = Runtime.getRuntime();
    Process process = null; 
    try{
      process = log.exec(_logfile);
      output  = new BufferedReader(new InputStreamReader
				    (process.getInputStream()));
      exit_status = process.waitFor();    
      process.destroy();
      exit_value = process.exitValue();      
      log.gc();

    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("WhoTool:gatherInformation: " + e);
    }
    catch(InterruptedException e){
      _status = "Error: " + e;
      System.err.println("WhoTool:gatherInformation: " + e);
    }

    try{
      while((this_line = output.readLine()) != null ){
        token = new StringTokenizer(this_line,delimitor);
        name  = new String(token.nextToken());
        token_vector.addElement(name);       
      }
      users = token_vector.lastElement();
      user  = users.toString();
      token = new StringTokenizer(user,"=");
      name  = new String(token.nextToken());
      user  = new String(token.nextToken());
      msngr = new String("USERS: " +user);

    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("WhoTool:gatherInformation: " + e);
    }   

    try{
      output.close();
    }
    catch(IOException e){
      _status = "Error: " + e;
      System.err.println("WhoTool: stringManipulator: " +e);
    }
    MessageI outputToMessageI = new MessageI(_key_store);
    Message param_data = new Message();
    
    String   doTool[] = {"m ", msngr, 
			 "h " ,_host_name,
			 "f " ,_logfile,  "s " ,"whotool"};
    outputToMessageI.parseParameters(doTool, param_data);
    outputToMessageI.validateParameters(param_data);
    outputToMessageI.openMessageConnection(_host_name);
    outputToMessageI.sendToHummer(param_data);
    outputToMessageI.closeMessageConnection();
  }
}