/*
Java Hummer Project
File: ConvertIP.java
Date: 11-23-98
Author: Jesse C. McConnell
Description:  Convert an IP address in the form XXX.XXX.XXX.XXX to the form
              XXXXXXXXXXXX so it can be compared for ranges.

Adapted from ConvertIP.h and ConvertIP.C from the Hummer Project.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util;

import java.net.InetAddress;

/**
Convert an IP address in the form XXX.XXX.XXX.XXX to the form XXXXXXXXXXXX
so it can be compared for ranges.  This is a hack job from the original
version of Hummer.  Exists as is until we take the time to come up with a
more Javaway of doing it.

@author Jesse C. McConnell
*/
public class ConvertIP {

  /**
  Takes in a InetAddress and returns a Long int representing the XXX.XXX.XXX.XXX
  address.  The address becomes XXXXXXXXXXXX.  0's should fill the locations
  that are not full XXX numbers.

  @param addr InetAddress to convert to a Long.
  @return The Long representing the InetAddress passed in.
  */
  public Long convertIP(InetAddress addr) {

    String value = new String();
    byte[] address = addr.getAddress();
    for (int i=0; i < address.length; i++) {
      value = value + value.valueOf(address[i] < 0 ? address[i] + 256 : address[i]);
    }
    try {
      Long ip = new Long(value);
      return ip;
    }
    catch (NumberFormatException e) {
      Long error = new Long(-1);
      return error;
    }
  }
};
