/*
Java Hummer Project
File: DefaultClientProtocol.java
Date: 4-5-99
Author: Chad Ward
Description:  Socket interface for a client-side connection of communication.
              This "default" version does not use any of the JCE classes, therefore
              it does do any two-way encryption/decryption.

 Copyright (C) 1997-1998 The Hummer Project - hummer@cs.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package edu.uidaho.hummer.util.socket;

/*
import java.io.IOException;
import java.io.OptionalDataException;
import java.io.ObjectOutputStream;
import java.io.ObjectInputStream;
import java.util.Date;
import java.util.Calendar;
import java.util.Vector;
import java.util.Hashtable;
import java.net.Socket;
import java.security.Key;
import java.security.PublicKey;
import java.security.PrivateKey;
import java.security.KeyPair;
import java.security.Signature;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.security.UnrecoverableKeyException;
import java.security.cert.CertificateException;
*/

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.Vector;
import java.security.Signature;
import java.security.PublicKey;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.KeyStoreException;
import java.security.InvalidKeyException;
import java.security.SignatureException;
import java.security.cert.CertificateException;

import edu.uidaho.hummer.dataobject.AliasInfo;
import edu.uidaho.hummer.dataobject.Response;
import edu.uidaho.hummer.dataobject.NewUserValidator;
import edu.uidaho.hummer.dataobject.PersonalConfig;
import edu.uidaho.hummer.dataobject.HummerSignedObject;
import edu.uidaho.hummer.dataobject.HummerCertificate;
import edu.uidaho.hummer.util.StaticMethods;
import edu.uidaho.hummer.util.HummerKeyStore;
import edu.uidaho.hummer.util.DefaultKeyStore;
import edu.uidaho.hummer.util.UnloadedKeyStoreException;
import edu.uidaho.hummer.util.UnknownAliasException;


/**
Concrete HummerProtocol instance for a client using
Signatures to authenticate itself to a server.  This method uses only classes
from the JDK 1.2 library, so encryption/decryption does not occur.

The following table describes the protocol between a DefaultServerProtocol
and a DefaultClientProtocol when both call their validation() methods.
Any tranmsissions following this are handled by the HummerSocket class. <br>

  <table border=1><tr><td>
  <b>For normal transmission</b>
  </td></tr><tr><td>
  <table border=0> <tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that client generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>For this first transmission, will be assumed to be null (ignored unless _type=Response._EXCEPTION)
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>String alias - The recognized alias for the account to loggin under
		</td></tr><tr><td VAlign=top>

  _data[2]</td><td VAlign=top>String signer_alias - Alias (IP address) of server that signed the certificate object (data[3])
		</td></tr><tr><td VAlign=top>

  _data[3]</td><td VAlign=top>HummerCertificate signature_certificate - Certificate signed by mutually known Hummer (data[2])
      </td></tr><tr><td VAlign=top>&nbsp;</td></tr><tr><td VAlign=top>
  <b> Response&nbsp;by&nbsp;Server:</b></td><td VAlign=top>
      Certificate is Valid, or requesting <i>alias</i> is for a local account, and the version
      of the HummerCertificate given can be verified as the most recent copy for the <i>alias</i>
		</td></tr><tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that server generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>A Long representation of message id that the client sent
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>HummerCertificate server_certificate - Certificate holding most recent PublicKey of server.
                                      It is with the same key that the client used to sign its HummerCertificate
                                      (Commonly trusted source).
		</td></tr><tr><td VAlign=top>
 #_data[2]</td><td VAlign=top>HummerCertificate new_client_certificate - If client is a local account, and its
                                      HummerCertificate was outdated, then a new HummerCertificate created
                                      and sent back to the client (can be verified with PublicKey in server_certificate).
   </td></tr><tr><td colspan=2>
   # Only when for local accounts, and when <i>alias'</i> HummerCertificate has expired
   </td></tr>
 </table></td></tr></table><br><br>

  <table border=1><tr><td VAlign=top>
  <b>For new accounts</b> Initial transmission by the client is the same, except, the <i>_data</i>
    Vector will hold the following:
		</td></tr><tr><td VAlign=top>
  <table border=0><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>String alias - The recognized alias for the account to loggin under
		</td></tr><tr><td VAlign=top>

  _data[2]</td><td VAlign=top>String signer_alias - null
		</td></tr><tr><td VAlign=top>

  _data[3]</td><td VAlign=top>client_public_key - PublicKey of the client wishing to initialize account
		</td></tr><tr><td VAlign=top>

  _data[4]</td><td VAlign=top>message_digest - Array of bytes representing a checksum like encoding
                                (java.security.MessageDigest) of the password for the account,
                                random_timestamp (_data[5]), and the encoded version of client_public_key
		</td></tr><tr><td VAlign=top>

  _data[5]</td><td VAlign=top>random_timestamp - Array of bytes representing a random number (double) and a
                                timestamp (long) that client generated and used for message_digest
      </td></tr><tr><td VAlign=top>&nbsp;</td></tr><tr><td VAlign=top>
  <b> Response&nbsp;by&nbsp;Server:</b></td><td VAlign=top>
      <i>alias</i> is for a new account, and message_digest is validated
		</td></tr><tr><td VAlign=top>
  int _type</td><td VAlign=top>(see Response._type) Should be equal to Response._OK
		</td></tr><tr><td VAlign=top>
  long _id</td><td VAlign=top>Long that server generated using (new Date()).getTime(), which will be used to give
              a "unique" id for each message.  This is done by incrementing the id for each new message.
              (Used to eliminate replay attacks)
		</td></tr><tr><td VAlign=top>
  Object _message</td><td VAlign=top>For this first transmission, will be assumed to be null 
                                     (ignored unless _type=Response._EXCEPTION)
		</td></tr><tr><td VAlign=top>
  Vector _data</td><td VAlign=top>Holder of major data.  See below values for contents of this Vector.
		</td></tr><tr><td VAlign=top>
  _data[0]</td><td VAlign=top>String protocol_to_use - The type of protocol to be used
                             (This must be "default" for DefaultServerSocket/DefaultClientServerSocket)
		</td></tr><tr><td VAlign=top>
  _data[1]</td><td VAlign=top>PublicKey server_public_key - The server's public key needed to verify signatures
		</td></tr><tr><td VAlign=top>
  _data[2]</td><td VAlign=top>HummerCertificate client_certificate - The client's new HummerCertificate
                                        which will be needed
                                        to log back into the server, and can be used to authenticate to other
                                        servers that trust the server that signed <i>client_certificate</i>
		</td></tr><tr><td VAlign=top>
  _data[3]</td><td VAlign=top>message_digest - Array of bytes representing a checksum like encoding
                                (java.security.MessageDigest) of the password for the account,
                                random_timestamp (_data[4]), and the encoded version of server_public_key   
		</td></tr><tr><td VAlign=top>

  _data[4]</td><td VAlign=top>random_timestamp - Array of bytes representing a random number (double) and a
                                timestamp (long) that client generated and used for message_digest
  		</td></tr>
  </table> </td></tr></table><br>
  For further documentation on protocol see the documentation for Hummer.
@author Chad Ward
*/
public final class DefaultClientProtocol extends HummerProtocol {

	/**
	Overrides the key store of the parent class, because
	protocol specific information is needed.
	*/
  private DefaultKeyStore _key_store;

	/**
	Alias used to sign certificate held by client.
	*/
  private String _signer_alias;

	/**
	Object for creating and verifying SignedObjects and HummerCertificates
	*/
  protected Signature _signing_signature_object = null, _verifying_signature_object;

	/**
	Certificate that client will use to login to the server.
	*/
  protected HummerCertificate _trusted_certificate;

	/**
	True if the PublicKey held by the server is not equal to the client's current
	PublicKey.
	*/
  protected boolean _must_update_public_key = false;

	/**
	Calls superclass constructor with the given parameters.
	@param empty_config This parameter is ignored.
		This is because there is never a password for the server, and the alias will always
		equal the IP address that the server is running on.
		This parameter is only to meet the requirements of the HummerSocket constructor
	@see HummerSocket#HummerSocket(boolean, PersonalConfig, Socket, HummerKeyStore)
	*/
  public DefaultClientProtocol(PersonalConfig user_config, HummerKeyStore key_store)
         throws NoSuchAlgorithmException, KeyStoreException, UnloadedKeyStoreException, UnknownHostException {
    super(user_config, key_store);
    this._name = "default";
    this._key_store = (DefaultKeyStore)key_store;
  }

	/**
	Sets the current partner alias and signer alias.
	*/
  public void initializeInformation(Object protocol_information, Object partner_alias)
         throws HummerSocketException {
    _current_partner_alias = partner_alias.toString();
    _signer_alias = (String)protocol_information;
    if(_signer_alias == null || _signer_alias.equals("null") ||
       _signer_alias.trim().equals("")) {
      _signer_alias = _current_partner_alias;
    }
  }

  public void checkBasicInputDataFormat(Object input_object)
         throws HummerSocketException, UnloadedKeyStoreException, KeyStoreException, UnknownAliasException {
    if(input_object == null) {
      throw new HummerSocketException("DefultClientProtocol:checkBasicInputDataFormat: " +
                                      "Null Response object recieved", null,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    HummerSignedObject signed_input = null;
    String object_type = input_object.getClass().getName();
    if(object_type.equals("edu.uidaho.hummer.dataobject.Response")) {
      _validation_response = (Response)(input_object);
      if(!_validation_response._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException")) {
        throw new HummerSocketException("DefaultClientProtocol:validation: " +
                       "Invalid type of _message in Response object " + object_type +
                       " (should be HummerSocketException)",
                       _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)_validation_response._message;
    }
    else if(!object_type.equals("edu.uidaho.hummer.dataobject.HummerSignedObject")) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                     "Invalid type of object recieved (" + object_type + ")",
                     null, HummerSocketException._INVALID_DATA_FORMAT);
    }
    signed_input = (HummerSignedObject)input_object;
    _validation_response = (Response)(signed_input.getObject());

    if(_verifying_signature_object == null) {
      if(_key_store.contains(_current_partner_alias) && !_key_store.isNewAccount(_current_partner_alias)) {
        _verifying_signature_object = _key_store.getVerifySignature(_current_partner_alias);
      }
      else if(_current_partner_alias.equals(_personal_config.getAlias())) {
        _verifying_signature_object = _key_store.getPersonalVerifySignature();
      }
      else if(this.isNewAccountValidation()) {
        PublicKey server_public_key = (PublicKey)(_validation_response._data.get(1));
        try {
          _verifying_signature_object = StaticMethods.getVerifyingSignatureObject(server_public_key);
        }
        catch(NoSuchAlgorithmException e) {
          throw new HummerSocketException("DefaultClientProtocol:checkBasicInputDataFormat: " +
                                          "System does not support signature algorithm: " + e,
                                          _validation_response, 0);
        }
        catch(InvalidKeyException e) {
          throw new HummerSocketException("DefaultClientProtocol:checkBasicInputDataFormat: " + e,
                                          _validation_response, 0);
        }
      }
      else {
        throw new HummerSocketException("DefaultClientProtocol:checkBasicInputDataFormat: " +
                                        "Could not retrieve information for current server(" +
                                        _current_partner_alias + "). " +
                                        "(Incorrect HummerKeyStore?)", _validation_response, 0);
      }
    }

    if(_validation_response._data == null) {
      throw new HummerSocketException("DefultClientProtocol:checkBasicInputDataFormat: " +
                                      "Null _data Vector recieved", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.size() < 1) {
      throw new HummerSocketException("DefultClientProtocol:checkBasicInputDataFormat: " +
                                      "Size of _data Vector must be non-zero", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    try {
      if(!signed_input.verify(_verifying_signature_object)) {
        throw new HummerSocketException("DefualtClientProtocol:checkBasicInputDataFormat: " +
                                        "Signed object recieved could not be verified with known server PublicKey.",
                                        _validation_response, HummerSocketException._INVALID_SIGNATURE);
      }
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultClientProtocol:checkBasicInputDataFormat: " + e,
                                      _validation_response, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultClientProtocol:checkBasicInputDataFormat: " + e,
                                      _validation_response, 0);
    }
  }

  public boolean isNewAccountValidation() {
    if(_personal_config == null) {
      _personal_config = new PersonalConfig(_signer_alias, null);
    }
    return _personal_config.isNewAccount();
  }

  public void initializedAccountCheckInputDataFormat() throws HummerSocketException {

    if(((_validation_response._type & Response._EXCEPTION) == Response._EXCEPTION)) {
      throw (HummerSocketException)(_validation_response._message);
    }
    if(_validation_response._data.size() < 2) {
      throw new HummerSocketException("DefultClientSocket:validation: " +
                                      "Size of _data Vector is too small", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(this._signer_alias.equals(_current_partner_alias)) {
		/*
		Local account login
		*/
      if(_validation_response._data.size() > 3) {
        throw new HummerSocketException("DefultClientSocket:validation: " +
                                        "Size of _data Vector is too large", _validation_response,
                                        HummerSocketException._INVALID_DATA_FORMAT);
      }
    }
    else {
		/*
		Remote account login
		*/
      if(_validation_response._data.size() > 2) {
        throw new HummerSocketException("DefultClientSocket:validation: " +
                                        "Size of _data Vector is too large", _validation_response,
                                        HummerSocketException._INVALID_DATA_FORMAT);
      }
    }
    if(_validation_response._data.get(0) == null) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                                      "Null protocol name recieved", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.get(1) == null) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                                      "Null HummerCertificate for server recieved", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    if(_validation_response._data.size() > 2 && _validation_response._data.get(2) == null) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                                      "Null replacement HummerCertificate for client recieved",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
  }

  public void initializedAccountVerifyInformation()
         throws HummerSocketException, UnknownAliasException, UnloadedKeyStoreException, KeyStoreException {
    _output_message_id = _validation_response._id;
    Signature signer_verifying_signature = null;
    if(_current_partner_alias.equals(_personal_config.getAlias()) &&
       _signer_alias.equals(_personal_config.getAlias())) {
      signer_verifying_signature = _key_store.getPersonalVerifySignature();
    }
    else {
      signer_verifying_signature = _key_store.getVerifySignature(this._signer_alias);
    }
    HummerCertificate server_certificate = null;
    try {
      server_certificate = (HummerCertificate)(_validation_response._data.get(1));
    }
    catch(ClassCastException e) {
      throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " +
                                      "Invalid class type recieved.  Expected: HummerCertificate.  Recieved: " +
                                      _validation_response._data.get(1).getClass().getName() + ".",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
		/*
		Catch exceptions recieved by the server, but only if the correct message id was placed on
		the message.
		*/
    if(((Long)(_validation_response._message)).longValue() != _input_message_id) {
      throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " +
                                      "Message ID recieved is not valid.",
                                      _validation_response, HummerSocketException._INVALID_MESSAGE_ID);
    }

    if(server_certificate.isExpired()) {
      throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " +
                                      "Expired certificate recieved.",
                                      _validation_response, HummerSocketException._EXPIRED_HUMMER_CERTIFICATE);
    }
    try {
      if(!server_certificate.isValid(signer_verifying_signature)) {
        throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " +
                                        "Invalid certificate recieved.",
                                        _validation_response, HummerSocketException._INVALID_LOGIN);
      }
    }
    catch(CertificateException e) {
      throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " + e,
                                      _validation_response, HummerSocketException._INVALID_LOGIN);
    }

    if(this._signer_alias.equals(_current_partner_alias) &&
       !_current_partner_alias.equals(_personal_config.getAlias())) {
      _key_store.setKey(AliasInfo._PUBLIC_KEY_TYPE, _current_partner_alias, server_certificate.getPublicKey());
      _verifying_signature_object = _key_store.getVerifySignature(_current_partner_alias);
      if(_validation_response._data.size() > 2) {
        try {
          HummerCertificate new_certificate = (HummerCertificate)(_validation_response._data.get(2));
          if(!new_certificate.isValid(_verifying_signature_object)) {
            throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " +
                                            "Invalid certificate recieved for " + _current_partner_alias + ".",
                                            _validation_response, HummerSocketException._INVALID_LOGIN);
          }
          _key_store.setHummerCertificate(_current_partner_alias,
                                          (HummerCertificate)(_validation_response._data.get(2)));
        }
        catch(CertificateException e) {
          throw new HummerSocketException("DefaultClientProtocol:initializedAccountVerifyInformation: " + e,
                                          _validation_response, HummerSocketException._INVALID_LOGIN);
        }
      }
    }
  }

  public Object initializedAccountPrepareResponse()
         throws HummerSocketException, UnloadedKeyStoreException, UnknownAliasException, KeyStoreException {
     if(!_current_partner_alias.equals(_personal_config.getAlias()) &&
        !_current_partner_alias.equals(_signer_alias) && !_key_store.contains(_signer_alias)) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                     "Could not find signer alias(" + _signer_alias + ") in HummerKeyStore (new account?).", null,
                     HummerSocketException._UNKNOWN_HUMMER_CERTIFICATE_SIGNER);
    }

    _input_message_id = StaticMethods.getUniqueTimestamp();
    Response output = new Response();
    output._type = Response._OK;
    output._id = _input_message_id;
    output._data = new Vector();
    output._data.add(this._name);
    output._data.add(_personal_config.getAlias());

    if(_personal_config.getAlias().equals(_current_partner_alias) ||
       _personal_config.getAlias().equals(_signer_alias)) {
      _trusted_certificate = _key_store.getPersonalHummerCertificate();
      _signing_signature_object = _key_store.getPersonalSignSignature();
    }
    else {
      _trusted_certificate = _key_store.getHummerCertificate(this._signer_alias);
      _signing_signature_object = _key_store.getPersonalSignSignature(_trusted_certificate.getPublicKey());
      if(!_trusted_certificate.getPublicKey().equals(_key_store.getPersonalKeyPair().getPublic())) {
        _must_update_public_key = true;
      }
    }

    output._data.add(this._signer_alias);
    output._data.add(_trusted_certificate);
    return output;
  }

  public Object initializedAccountFinalize() throws HummerSocketException, UnloadedKeyStoreException {
    if(_must_update_public_key) {
      Response output = null;
      output = new Response();
      output._type = Response._OK | Response._UPDATE_PUBLIC_KEY;
      output._data.add(_key_store.getPersonalKeyPair().getPublic());
      return prepareOutput(output);
    }
    return null;
  }



  public void newAccountCheckInputDataFormat() throws HummerSocketException {
    if(_validation_response._data.size() != 5) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Size of Response._data Vector must be 5.", _validation_response, 0);
    }

    if(_validation_response._data.get(0) == null) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Null protocol value recieved", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(1) == null) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Null PublicKey for server recieved",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(2) == null) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Null HummerCertificate for client recieved",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(3) == null) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Null message digest recieved", _validation_response,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }

    if(_validation_response._data.get(4) == null) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " +
                                      "Null random value and timestamp recieved",
                                      _validation_response, HummerSocketException._INVALID_DATA_FORMAT);
    }
  }



  public void newAccountVerifyInformation()
         throws HummerSocketException, KeyStoreException, UnknownAliasException, UnloadedKeyStoreException {
    if(((Long)(_validation_response._message)).longValue() != _input_message_id) {
      throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " +
                                      "_message in Response object is not equal to message ID sent",
                                      _validation_response, 0);
    }

    if(!_validation_response._data.get(0).toString().equals(this._name)) {
      throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " +
                     "Invalid protocol request.", _validation_response, HummerSocketException._PROTOCOL_NOT_KNOWN);
    }

    _output_message_id = _validation_response._id;
    PublicKey server_public_key = null;
    byte[] message_digest = null, generated_digest = null, random_timestamp = null;
    try {
      server_public_key = (PublicKey)(_validation_response._data.get(1));
      message_digest = (byte[])(_validation_response._data.get(3));
      random_timestamp = (byte[])(_validation_response._data.get(4));
      generated_digest = StaticMethods.getDigest(_personal_config.getPassword(),
                                               random_timestamp, server_public_key);
    }
    catch(ClassCastException e) {
      throw new HummerSocketException("HummerClientProtocol:newAccountVerifyInformation: " + e, null,
                                      HummerSocketException._INVALID_DATA_FORMAT);
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("HummerClientProtocol:newAccountVerifyInformation: " + e, null, 0);
    }
    if(!MessageDigest.isEqual(message_digest, generated_digest)) {
      throw new HummerSocketException("HummerClientProtocol:newAccountValidation: " +
                                      "Invalid MessageDigest sent.", _validation_response, 0);
    }
    if(!_key_store.contains(_current_partner_alias)) {
      _key_store.addServerAlias(_current_partner_alias, null, false, null);
      try {
        _key_store.initializeAccount(_current_partner_alias,
                                     StaticMethods.getDigest(null, null, server_public_key),
                                     null, server_public_key);
      }
      catch(NoSuchAlgorithmException e) {
        throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " + e,
                                                                    null, 0);
      }
      catch(CertificateException e) {
        throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " + e,
                                                                    _validation_response, 0);
      }
    }
    else {
      if(_key_store.isNewAccount(_current_partner_alias)) {
        try {
          _key_store.resetAlias(_current_partner_alias, null);
          message_digest = StaticMethods.getDigest(null, null, server_public_key);
          _key_store.initializeAccount(_current_partner_alias, message_digest, null, server_public_key);
        }
        catch(NoSuchAlgorithmException e) {
          throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " + e,
                                                                      _validation_response, 0);
        }
        catch(CertificateException e) {
          throw new HummerSocketException("HummerClientProtocol: newAccountVerifyInformation: " + e,
                                                                      _validation_response, 0);
        }
      }
      else if(!_current_partner_alias.equals(_personal_config.getAlias())) {
        _key_store.setKey(AliasInfo._PUBLIC_KEY_TYPE, _current_partner_alias, server_public_key);
      } 
    }
    if(!_current_partner_alias.equals(_personal_config.getAlias())) {
      try {
        _key_store.setHummerCertificate(_current_partner_alias,
                                        (HummerCertificate)(_validation_response._data.get(2)));
        _verifying_signature_object = _key_store.getVerifySignature(_current_partner_alias);
      }
      catch(CertificateException e) {
          throw new HummerSocketException("DefaultClientProtocol:newAccountValidation: " + e, _validation_response, 0);
      }
    }

    _signing_signature_object = _key_store.getPersonalSignSignature();
    _personal_config.initializeAccount();
  }

  public Object newAccountPrepareResponse() throws HummerSocketException, UnloadedKeyStoreException {
      if(_personal_config == null) {
      throw new HummerSocketException("DefaultClientProtocol:validation: " +
                     "PersonalConfig object recieved is null", null,
                     HummerSocketException._INVALID_DATA_FORMAT);
    }
    _input_message_id = StaticMethods.getUniqueTimestamp();
    Response output = new Response();
    output._type = Response._OK;
    output._id = _input_message_id;
    output._data = new Vector();
    output._data.add(this._name);
    output._data.add(_personal_config.getAlias());
    output._data.add(null);
    try {
      PublicKey personal_public_key = _key_store.getPersonalKeyPair().getPublic();
      byte[] random_timestamp = StaticMethods.getRandomTimestampBytes(),
             message_digest = null;

      message_digest = StaticMethods.getDigest(_personal_config.getPassword(),
                                               random_timestamp, personal_public_key);
      output._data.add(personal_public_key);
      output._data.add(message_digest);
      output._data.add(random_timestamp);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountPrepareResponse: " + e,
                                      null, 0);
    }
    catch(NoSuchAlgorithmException e) {
      throw new HummerSocketException("DefaultClientProtocol:newAccountPrepareResponse: " + e,
                                      null, 0);
    }
    return output;
  }

  public Object newAccountFinalize() { return null; }

  public Response verifyInput(Object input_object) throws HummerSocketException  {
    Response input = null;
    if(input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.Response")) {
      input = (Response)input_object;
      if(input._message == null) {
        throw new HummerSocketException( "DefaultClientProtocol:verifyInput: " +
                                        "Read Response object (meaning exception thrown), but _message is null.",
                                        null, HummerSocketException._INVALID_DATA_FORMAT);
      } else if(!input._message.getClass().getName().equals("edu.uidaho.hummer.util.socket.HummerSocketException")) {
        throw new HummerSocketException("DefaultClientProtocol:verifyInput: " +
                                        "Read Response object of type _EXCEPTION, but _message is not of " +
                                         "type HummerSocketException.",
                                        null, HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)input._message;
    }
    else if(!input_object.getClass().getName().equals("edu.uidaho.hummer.dataobject.HummerSignedObject")) {
      throw new HummerSocketException("DefaultClientProtocol:verifyInput: " +
                                 "Invalid type of object recieved from input stream.  Should be HummerSignedObject.",
                                 null, HummerSocketException._INVALID_DATA_FORMAT);
    }
    HummerSignedObject signed_object = (HummerSignedObject)input_object;
    boolean signed_object_valid = false;
    try {
      signed_object_valid = signed_object.verify(_verifying_signature_object);
      input = (Response)(signed_object.getObject());
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultClientProtocol:verifyInput: " + e, input, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultClient:verifyInput: " + e, input, 0);
    }
    if((input._type & Response._EXCEPTION) == Response._EXCEPTION) {
      if(input._message.getClass().getName().equals("java.lang.String")) {
        input._message = new HummerSocketException((String)input._message,
                                                   null, HummerSocketException._INVALID_DATA_FORMAT);
      }
      throw (HummerSocketException)(input._message);
    }
    else if((input._type & Response._OK) == 0) {
        throw new HummerSocketException("DefaultClientProtocol:readResponseObject: " +
                                      "Invalid _type recieved on Response object (" + input._type + ").",
                                      input, HummerSocketException._INVALID_DATA_FORMAT);
    }
    _input_message_id++;
    if(input._id != _input_message_id) {
      throw new HummerSocketException("HummerClientProtocol:verifyInput: " +
                                      "Incorrect message ID recieved", input, 0);
    }
    try {
      if(!signed_object.verify(_verifying_signature_object)) {
        throw new HummerSocketException("DefualtClientProtocol:verifyInput: " +
                                        "Signed object recieved could not be verified with server PublicKey.",
                                        _validation_response, HummerSocketException._INVALID_SIGNATURE);
      }
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultClientProtocol:verifyInput: " + e, input, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultClient:verifyInput: " + e, input, 0);
    }
    return input;
  }

  public HummerProtocol updateProtocol() throws HummerSocketException {
    String requested_protocol = _validation_response._data.get(0).toString();
    if(!requested_protocol.equals("default")) {
      throw new HummerSocketException("DefaultClientProtocol:updateProtocol: " +
                                      "Cannot connect with protocol: " + _validation_response,
                                      null, HummerSocketException._PROTOCOL_NOT_KNOWN);
    }
    else {
      return this;
    }
  }

  public Object prepareOutput(Response write_object) throws HummerSocketException {
    if(write_object == null) {
      write_object = new Response();
      write_object._message = null;
      write_object._data = null;
    }
    _output_message_id++;
    write_object._id = _output_message_id;
    HummerSignedObject signed_object = null;
    try {
      signed_object = new HummerSignedObject(write_object, _signing_signature_object);
    }
    catch(SignatureException e) {
      throw new HummerSocketException("DefaultClientProtocol:prepareOutput: " +
                                      "System error when generating HummerSignedObject.  " + e, null, 0);
    }
    catch(IOException e) {
      throw new HummerSocketException("DefaultClientProtocol:prepareOutput: " +
                                      "System error when generating HummerSignedObject.  " + e, null, 0);
    }
    return signed_object;
  }
}
