/*
Java Hummer Project
File: DummyClient.java
Date: 2-15-99
Author: Joel Marlow
Description:  basic client

 Copyright (C) 1997-1998 The Hummer Project - hummer@csds.uidaho.edu
   
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
     
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

import java.net.*;
import java.io.*;
import java.util.*;
import java.sql.*;
import edu.uidaho.hummer.util.*;
import edu.uidaho.hummer.util.socket.*;
import edu.uidaho.hummer.dataobject.*;

/**
This class tests that the information is put into and received from the database
through the Configuration Server
 */
public class DummyClient { 

  /**
  Database Url which is taken from the corresponding _constants property.
  */
  private String _config_url = new String();
  /**
  Database Driver which is taken from the corresponding _constants property.
  */
  private String _sql_driver = new String();
  /**
  Database user which us taken from the corresponding _constants property.
  */
  private String _sql_user = new String();
  /**
  Database password which is taken from the corresponding _constants property.
  */
  boolean _made_connection = false;
  private String _sql_pwd = new String(); 
  private Socket _client_socket;
  private Properties _constants = new Properties(System.getProperties());
  private int _port = 16000;
  Config _config = new Config();  
  InetAddress _sub_addr = null;
  //palouse = 129.101.55.49 this is used to check the peers subscribtion
  //  String _peer_server = new String("129.101.55.49");
  //phobos = 129.101.54.54 this is used to check the peers subscribtion
  //  String _peer_server = new String("129.101.54.54");
  //europa = 129.101.54.53 this is used to check the peers subscribtion
  //  String _peer_server = new String("129.101.54.53");
  //++Needed for authentication
  HummerKeyStore _key_store;
  //++Needed for authentication
  PersonalConfig _personal;
  //++Needed for authentication
  HummerClientSocket _dcs = null;

  public DummyClient(String server_name) {
    String address = new String();

    //++This try and catch is needed for authentication
    try {
      address = InetAddress.getLocalHost().getHostAddress();
      _key_store = new HummerKeyStore();
      _key_store.load(new FileInputStream(address +"_key.hum"));
    }
    catch (Exception e) {
      System.err.println("DummyClient: " + e);
      //e.printStackTrace();
      //System.exit(-1);
    }
  //++Needed for authentication
    _personal = new PersonalConfig(address, "test".getBytes());
    //    _personal = new PersonalConfig("admin", null);

    try {
      _sub_addr = InetAddress.getByName(server_name);
    }
    catch(UnknownHostException e){
      System.err.println("DummyClient " + e);
    }

    insertSQL();
    openServer(_sub_addr);
    if (_made_connection) {
      validateConnection();
      sendData();
      checkSQL();
      insertMore();
      sendData();
      checkSQL();
      insertClose();
      sendData();
      checkSQL();
  //++This try and catch is needed for authentication it should only be used
  //++if the users account is being initialized
      try {
        _key_store.store(new FileOutputStream(address +"_key.hum"));
        _dcs.close();
      }
      catch (Exception e) {           
        System.err.println("DummyClient: " + e);
      }     

    }
    else {
      System.out.println("There needs to be a DummyServer running on " + _sub_addr.getHostName());
    }
  }

  public boolean openServer(InetAddress sub_name) {
    // Create new socket 
    try  {
      _client_socket = new Socket(sub_name, _port);
      System.out.println("Connected to chat server.");
      _made_connection = true;
    } 
    catch(UnknownHostException e) {
      System.err.println("DummyClient:openServer:Host not found:  " + e.getMessage());
      _made_connection = false;

    }
    catch(IOException e) {
      System.err.println("DummyClient:openServer: " + e.getMessage());
      _made_connection = false;
    }
    return true;
  } //openServer

  /**
  This method sends the data to the server and closes the _client_socket
  to the subordinate.

  @return boolean to say whether it executed correctly, currently 
  this is not used for anything. 
  */
  public boolean validateConnection() {

  //++This try and catch is needed for authentication
    try {
      _dcs = new HummerClientSocket(_personal, _client_socket, _key_store);
      //      System.out.println("before validation");

      
      String user_input = null;
      BufferedReader input_stream = new BufferedReader(new InputStreamReader(System.in));
      while(user_input == null || (!user_input.equals("y") && !user_input.equals("n"))) {
        System.out.println("is new account (y/n):");
        user_input = input_stream.readLine().toLowerCase();
      }

      if(user_input.equals("n")) {
        _personal.initializeAccount();
      }
      //    _dcs.validation("129.101.54.53"); this should be the same as below
      _dcs.validation(null);
    }
    catch (Exception e) {
      e.printStackTrace();
      System.err.println("DummyClient:validateConnection: " + e);
      //      System.out.println("I have an error here!");
    }
    return true;
  }//validateConnection

  public void sendData() {

  //++Needed for authentication
    Response response = new Response();
  //++Needed for authentication
    response._type = Response._OK;
  //++Needed for authentication
    response._message = _config;
    // End of protocol mesage, and close out connection
  //++This try and catch is needed for authentication
    try  {
      _dcs.writeResponseObject(response);
      //System.out.println("first");
      response = _dcs.readResponseObject();
      //System.out.println("second");
      _config = (Config)response._message;
    }
    catch(UnknownHostException e) {
        System.err.println("DummyClient:sendData:Host not found:  " + e.getMessage());
        System.exit(1);
    }
    catch(IOException e) {
      System.err.println("DummyClient:sendData: " + e.getMessage());
      System.exit(1);
    }
    catch (Exception e) {
      System.err.println("DummyClient:sendData: " + e);
      e.printStackTrace();
    }
  } //sendData


  public void insertSQL() {
    String query = new String();

    query = "Did you get this message? ";
    _config.data.addElement(query);    
  }

  public void insertMore() {
    String query = new String();

    _config.data = new Vector();
    query = "This should be another message? ";
    _config.data.addElement(query);    
  }

  public void insertClose() {
    String query = new String();

    _config.data = new Vector();
    query = "close";
    _config.data.addElement(query);    
  }

  public void checkSQL() {
    Enumeration enum = _config.data.elements();
    Enumeration table_enum;
    Enumeration row_enum;
    Vector table = new Vector();
    Vector row = new Vector();
    boolean result = false;

    System.out.println(enum.nextElement());
    /*    table = (Vector)enum.nextElement();
    table_enum = table.elements();
    while (table_enum.hasMoreElements()) {
      row = (Vector)table_enum.nextElement();
      row_enum = row.elements(); 
      while (row_enum.hasMoreElements()) {
        System.out.print(row_enum.nextElement() + " ");
      }
      System.out.println();
    }
    */

  }//checkSQL

  public static void main(String[] args) throws Exception {
    //phobos = 129.101.54.54 this configuration sever needs to be up
    //    DummyClient cs = new DummyClient("129.101.54.54");
    //europa = 129.101.54.53 this configuration sever needs to be up
    //    DummyClient cs = new DummyClient("129.101.54.53");
    //kraken = 129.101.53.33 this configuration sever needs to be up
    //DummyClient cs = new DummyClient(InetAddress.getLocalHost().getHostAddress());
    //palouse = 129.101.55.49 this configuration sever needs to be up
    if(args.length == 0) {
      DummyClient cs = new DummyClient(InetAddress.getLocalHost().getHostAddress());
    }
    else {
      DummyClient cs = new DummyClient(args[0]);
    }
  }

}
