#!/usr/local/bin/perl

##########################################################################
# Hummer Tester
# Writen by:  Matt McDonald
# 1/8/98
#
# Purpose:  This program is the main program used for testing
# the Hummer.  It reads in, from a Test[var].dat file the (1) Test running,
# (2) the Server that the test uses, (3) the Table used, (4) and the
# message that is the basic command line being sent to the server.  It
# then checks for the correct server using the name from the .dat file 
# and runs it.  The specific sub program will then call the tester()
# which connects to the database and checks the correct table to see if
# the entries in it match the Test[var].exp file.  This .exp file should
# be created by the person(s) testing.  For more information on the 
# correct way to set up the different file see the README file.
########################################################################## 

use Pg;
use Sys::Hostname;

$host = hostname();              # gets the name of the host

my $test; 			 # stores the test name that's running
my $server;			 # stores the correct server name
my $message;			 # stores the message for the command line

# This loop checks the directory for every file that is a .dat file 
# and runs the specific test within it.  The tests should be named 
# in the form of Test[var].dat.

while(defined($filename = glob("./Test/*.dat"))) {          
  print "\n*************\nRunning $filename\n";

  open(FILE, $filename) || die "Can't open file: #!";     # opens the .dat file
  $test = <FILE>;                                         # reads in test name
  chomp($test);
  $server = <FILE>;                                       # reads in server name
  chomp($server);
  $tableused = <FILE>;                                    # reads in table used
  chomp($tableused); 
  $message = <FILE>;                                      # reads in message
  chomp($message);

  # makes a connection to the database and removes everything from the table 
  my $connect = Pg::connectdb("dbname = hummer_$host") || die " Couldn't connect: $!";
  $result = $connect->exec("delete from $tableused;");

  # opens the TestResults file and deletes everything within
  `rm TestResults`;
  `touch TestResults`;

  # nested if's check for and call the correct server
  if($server eq "ToolI") { 
    toolITest(); 
  }
  else {
    if($server eq "MessagingServer") {
      messagingServerTest();
    }
    else {
      if($server eq "ConfigServer") {
        configServerTest();
      }
      else {
        if($server eq "PeerServer") {
          peerServerTest();
        }
      }
    }
  }
}

#################
# Tests the ToolI server in the hummer
#################
sub toolITest {

  print "Testing: $server\n";    

  `java ToolI $message`; 		#command line using $message
  sleep 10;				#waits for db update
  tester();				#calls tester()
}

#################
# Tests the MessagingServer
#################
sub messagingServerTest {

  print "Testing: $server\n";

  `java MessagingServer $message`;	#command line using $message
  sleep 10;				#waits for db update
  tester();				#calls tester()
}

#################
# Tests the ConfigServer
#################
sub configServerTest {

  print "Testing: $server\n";

  `java ConfigServer $message`;  	#command line using $message
  sleep 10;				#waits for db update
  tester();				#calls tester()
}

#################
# Tests the PeerServer
#################
sub peerServerTest {

  print "Testing: $server\n";

  `java PeerServer $message`;		#command line using $message
  sleep 10;				#waits for db update
  tester();				#calls tester()
}

################
# Tests if the results in the database are the same as the ones
# in the .exp file from the above test runs
################
sub tester {

  my $final = 0;               # flag used to see if errors are found

  # opens the expected file from Test[val].exp  
  open(TESTFILE, "./Test/$test.exp") || die "Can't open test.exp file: $!";
  $table = <TESTFILE>;		# reads the table used
    chomp($table);
  $rows = <TESTFILE>;		# reads in the rows in table
    chomp($rows);
  $cols = <TESTFILE>;		# reads in the columns in table
    chomp($cols);
 
  # cuts the resuls wherever there is a blank space and puts each value in
  # an array called expect
  for($l = 0; $l <= $rows; $l++) {
    $check = <TESTFILE>;          # reads in the results that are expected
      chomp($check);
    @temparray = split / /, $check;
    $expect[$l] = [@temparray];
  }

  # connects to table in database for reading
  my $conn = Pg::connectdb("dbname = hummer_$host") || die " Couldn't connect: $!";
  $result = $conn->exec("select * from $table;");

  # opens the TestResults file for appending
  open(TESTRESULTS, ">>TestResults") || die "Can't open TestResults file: $!";
  print TESTRESULTS "\n$test:\n";
  
  $k = 1;
  for($i = 0; $i <= $rows; $i++) {       	# for every row in table
    for($j = 0; $j <= $cols; $j++) {		# for every col in table
      $answer = $result->getvalue($i, $j);	# reads in val of ($i, $j)
      if($expect[$i][$j] ne $answer) { 		# checks if expected not equal to table val
        $final = 1;				# sets flag
        $goOut = "$k: Incorrect data check with table: $table at position ($i, $j)";

        # writes error to TestResults file 
        print TESTRESULTS "$goOut\n";   
        print TESTRESULTS "   Expected: $expect[$i][$j]   Actual: $answer\n";
        $k++;
      }
     
    }
  }

  if($final eq 1) {					# writes error to monitor	
    print "ERROR: see $test in TestResults\n";
  }
  else {
    print "$test is CORRECT\n";			   	# writes correct to monitor
    print TESTRESULTS "$test is CORRECT";
  }
  close(TESTRESULTS);					# closed the appended file
  
}

print "\n";

